# $Id: synchronization.py 2013 2005-12-31 03:19:39Z zzzeek $
# synchronization.py - synchronization functions for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#

__all__  = ["Synchronizer", "NameLock", "_threading", "_thread"]


import os, weakref, tempfile, re, sys
from myghty.util import *

try:
    import thread as _thread
    import threading as _threading
except ImportError:
    import dummy_thread as _thread
    import dummy_threading as _threading

# check for fcntl module
try:
    sys.getwindowsversion()
    has_flock = False
except:
    try:
        import fcntl
        has_flock = True
    except ImportError:
        has_flock = False


class NameLock:
    """a proxy for an RLock object that is stored in a name 
    based registry.  
    
    Multiple threads can get a reference to the same RLock based on 
    the name alone, and synchronize operations related to that name.
    """
     
    locks = WeakValuedRegistry()

    class NLContainer:
        """cant put Lock as a weakref"""
        def __init__(self, reentrant):
            if reentrant:
                self.lock = _threading.RLock()
            else:
                self.lock = _threading.Lock()
        def __call__(self):
            return self.lock

    def __init__(self, identifier = None, reentrant = False):
        self.lock = self._get_lock(identifier, reentrant)

    def acquire(self, wait = True):
        return self.lock().acquire(wait)

    def release(self):
        self.lock().release()

    def _get_lock(self, identifier, reentrant):
        
        if identifier is None:
            return NameLock.NLContainer(reentrant)
        
        return NameLock.locks.get(identifier, lambda: NameLock.NLContainer(reentrant))



synchronizers = WeakValuedRegistry()

    
def Synchronizer(identifier = None, use_files = False, lock_dir = None, digest_filenames = True):
    """
    returns an object that synchronizes a block against many simultaneous 
    read operations and several synchronized write operations. 

    Write operations
    are assumed to be much less frequent than read operations,
    and receive precedence when they request a write lock.

    uses strategies to determine if locking is performed via threading objects
    or file objects.
    
    the identifier identifies a name this Synchronizer is synchronizing against.
    All synchronizers of the same identifier will lock against each other, within
    the effective thread/process scope.
    
    use_files determines if this synchronizer will lock against thread mutexes
    or file locks.  this sets the effective scope of the synchronizer, i.e. 
    it will lock against other synchronizers in the same process, or against
    other synchronizers referencing the same filesystem referenced by lock_dir.
    
    the acquire/relase methods support nested/reentrant operation within a single 
    thread via a recursion counter, so that only the outermost call to 
    acquire/release has any effect.  
    """

    if not has_flock:
        use_files = False
        
    if use_files:
        # FileSynchronizer is one per thread
        return synchronizers.sync_get("file_%s_%s" % (identifier, _thread.get_ident()), lambda: FileSynchronizer(identifier, lock_dir, digest_filenames))
    else:
        # ConditionSynchronizer is shared among threads
        return synchronizers.sync_get("condition_%s" % identifier, lambda: ConditionSynchronizer(identifier))


class SyncState:
    """used to track the current thread's reading/writing state as well as reentrant block counting"""
    
    def __init__(self):
        self.reentrantcount = 0
        self.writing = False
        self.reading = False

class SynchronizerImpl(object):
    """base for the synchronizer implementations.  the acquire/release methods keep track of re-entrant
    calls within the current thread, and delegate to the do_XXX methods when appropriate."""
    
    def __init__(self, *args, **params):
        pass

    def release_read_lock(self):
        state = self.state

        if state.writing: raise "lock is in writing state"
        if not state.reading: raise "lock is not in reading state"
        
        if state.reentrantcount == 1:
            self.do_release_read_lock()
            state.reading = False

        state.reentrantcount -= 1
        
    def acquire_read_lock(self, wait = True):
        state = self.state

        if state.writing: raise "lock is in writing state"
        
        if state.reentrantcount == 0:
            x = self.do_acquire_read_lock(wait)
            if (wait or x):
                state.reentrantcount += 1
                state.reading = True
            return x
        elif state.reading:
            state.reentrantcount += 1
            return True
            
    def release_write_lock(self):
        state = self.state

        if state.reading: raise "lock is in reading state"
        if not state.writing: raise "lock is not in writing state"

        if state.reentrantcount == 1:
            self.do_release_write_lock()
            state.writing = False

        state.reentrantcount -= 1
        
    def acquire_write_lock(self, wait  = True):
        state = self.state

        if state.reading: raise "lock is in reading state"
        
        if state.reentrantcount == 0:
            x = self.do_acquire_write_lock(wait)
            if (wait or x): 
                state.reentrantcount += 1
                state.writing = True
            return x
        elif state.writing:
            state.reentrantcount += 1
            return True

    def do_release_read_lock():raise NotImplementedError()
    def do_acquire_read_lock():raise NotImplementedError()
    def do_release_write_lock():raise NotImplementedError()
    def do_acquire_write_lock():raise NotImplementedError()
    
class FileSynchronizer(SynchronizerImpl):
    """a synchronizer using lock files.   as it relies upon flock(), which
    is not safe to use with the same file descriptor among multiple threads (one file descriptor
    per thread is OK), 
    a separate FileSynchronizer must exist in each thread."""
    
    def __init__(self, identifier, lock_dir, digest_filenames):
        self.state = SyncState()

        if lock_dir is None:
            lock_dir = tempfile.gettempdir()
        else:
            lock_dir = lock_dir

        self.encpath = EncodedPath(lock_dir, [identifier], extension = '.lock', digest = digest_filenames)
        self.filename = self.encpath.path

        self.opened = False
        self.filedesc = None
    
    def _open(self, mode):
        if not self.opened:
            try:
                self.filedesc = os.open(self.filename, mode)
            except OSError, e:
                self.encpath.verify_directory()
                self.filedesc = os.open(self.filename, mode)
            self.opened = True
            
    def do_acquire_read_lock(self, wait):
        self._open(os.O_CREAT | os.O_RDONLY)

        if not wait:
            try:
                fcntl.flock(self.filedesc, fcntl.LOCK_SH | fcntl.LOCK_NB)
                ret = True
            except IOError:
                ret = False
                
            return ret
        else:
            fcntl.flock(self.filedesc, fcntl.LOCK_SH)
            return True
        
        
    def do_acquire_write_lock(self, wait):
        self._open(os.O_CREAT | os.O_WRONLY)

        if not wait:
            try:
                fcntl.flock(self.filedesc, fcntl.LOCK_EX | fcntl.LOCK_NB)
                ret  = True
            except IOError:
                ret = False
                
            return ret
        else:
            fcntl.flock(self.filedesc, fcntl.LOCK_EX);
            return True
    
    def do_release_read_lock(self):
        self.release_all_locks()
    
    def do_release_write_lock(self):
        self.release_all_locks()
    
    def release_all_locks(self):
        if self.opened:
            fcntl.flock(self.filedesc, fcntl.LOCK_UN)
            os.close(self.filedesc)
            self.opened = False

    def __del__(self):
        if os.access(self.filename, os.F_OK):
            try:
                os.remove(self.filename)
            except OSError:
                # occasionally another thread beats us to it
                pass                    


class ConditionSynchronizer(SynchronizerImpl):
    """a synchronizer using a Condition.  this synchronizer is based on threading.Lock() objects and
    therefore must be shared among threads."""
    
    def __init__(self, identifier):
        self.tlocalstate = ThreadLocal(creator = lambda: SyncState())

        # counts how many asynchronous methods are executing
        self.async = 0

        # pointer to thread that is the current sync operation
        self.current_sync_operation = None

        # condition object to lock on
        self.condition = _threading.Condition(_threading.Lock())

    state = property(lambda self: self.tlocalstate())
        
    def do_acquire_read_lock(self, wait = True):    
        self.condition.acquire()

        # see if a synchronous operation is waiting to start
        # or is already running, in which case we wait (or just
        # give up and return)
        if wait:
            while self.current_sync_operation is not None:
                self.condition.wait()
        else:
            if self.current_sync_operation is not None:
                self.condition.release()
                return False

        self.async += 1
        
        self.condition.release()

        if not wait: return True
        
    def do_release_read_lock(self):
        self.condition.acquire()

        self.async -= 1
        
        # check if we are the last asynchronous reader thread 
        # out the door.
        if self.async == 0:
            # yes. so if a sync operation is waiting, notifyAll to wake
            # it up
            if self.current_sync_operation is not None:
                self.condition.notifyAll()
        elif self.async < 0:
            raise "Synchronizer error - too many release_read_locks called"
            
        self.condition.release()

    
    def do_acquire_write_lock(self, wait = True):
        self.condition.acquire()

        # here, we are not a synchronous reader, and after returning,
        # assuming waiting or immediate availability, we will be.
        
        if wait:
            # if another sync is working, wait
            while self.current_sync_operation is not None:
                self.condition.wait()
        else:
            # if another sync is working,
            # we dont want to wait, so forget it
            if self.current_sync_operation is not None:
                self.condition.release()
                return False
            
        # establish ourselves as the current sync 
        # this indicates to other read/write operations
        # that they should wait until this is None again
        self.current_sync_operation = _threading.currentThread()

        # now wait again for asyncs to finish
        if self.async > 0:
            if wait:
                # wait
                self.condition.wait()
            else:
                # we dont want to wait, so forget it
                self.current_sync_operation = None
                self.condition.release()
                return False
        
        self.condition.release()
        
        if not wait: return True

    def do_release_write_lock(self):
        self.condition.acquire()


        if self.current_sync_operation != _threading.currentThread():
            raise "Synchronizer error - current thread doesnt have the write lock"

        # reset the current sync operation so 
        # another can get it
        self.current_sync_operation = None

        # tell everyone to get ready
        self.condition.notifyAll()

        # everyone go !!
        self.condition.release()





# test/component_encoding.py
"""Test that component's encodings are determined correctly.
"""
import unittest, os, shutil, sys, warnings, random, StringIO, time, codecs, re

import testbase
from myghty.interp import Interpreter
from myghty.resolver import NotFound
from myghty import importer, exception

some_encodings = """ascii
                    big5 big5hkscs
                    cp037 cp424 cp437 cp500 cp737 cp775 cp850
                    cp852 cp855 cp856 cp857 cp860 cp861 cp862
                    cp863 cp864 cp865 cp866 cp869 cp874 cp875
                    cp932 cp949 cp950 cp1006 cp1026 cp1140 cp1250
                    cp1251 cp1252 cp1253 cp1254 cp1255 cp1256 cp1257 cp1258
                    euc_jp euc_jis_2004 euc_jisx0213 euc_kr
                    gb2312 gbk gb18030 hz
                    iso2022_jp iso2022_jp_1 iso2022_jp_2 iso2022_jp_2004
                    iso2022_jp_3 iso2022_jp_ext
                    iso2022_kr
                    latin_1
                    iso8859_2 iso8859_3 iso8859_4 iso8859_5 iso8859_6
                    iso8859_7 iso8859_8 iso8859_9 iso8859_10 iso8859_13
                    iso8859_14 iso8859_15
                    johab
                    koi8_r koi8_u
                    mac_cyrillic mac_greek mac_iceland
                    mac_latin2 mac_roman mac_turkish
                    ptcp154
                    shift_jis shift_jis_2004 shift_jisx0213
                    utf_16 utf_16_be utf_16_le utf_7""".split()

def isAsciiSuperset(encoding):
    ascii_chars = ''.join(map(chr, range(0x80)))
    try:
        return ascii_chars.decode(encoding) == ascii_chars.decode('ascii')
    except (ValueError, LookupError):
        return False

some_encodings = [ enc for enc in some_encodings
                   if isAsciiSuperset(enc) and enc != sys.getdefaultencoding()
                   ]
random.shuffle(some_encodings)

def random_encoding():
    enc = some_encodings.pop()
    some_encodings.insert(0, enc)
    return enc

def parse_magic_comment(text):
    m = re.match(r'(?:#.*\n)?#.*coding[:=][ \t]*([-\w.]+)', text)
    if m:
        return m.group(1)
    return None
        
def resetwarnings():
    warnings.resetwarnings()
    try:
        import myghty.compiler
        del myghty.compiler.__warningregistry__
    except AttributeError:
        pass

        
class FileComponentEncodingTests(testbase.ComponentTester):
        
    def getComponent(self, head=""):
        return self.makeFileBasedComponent(head + "\nbody\n")

        
    def getCompiled(self, head=""):
        """Get the python source for the compiled component.
        """
        return self._getCompiledFromComp(self.getComponent(head))

    def _getCompiledFromComp(self, comp):
        return file(comp.component_source.module.__file__).read()
        
    def failUnlessComponentEncodingEqual(self, comp, encoding):
        #self.failUnlessEqual(comp.component_source.module._ENCODING,
        #                     Encoding(encoding))
        compiled = self._getCompiledFromComp(comp)
        enc = parse_magic_comment(compiled) or sys.getdefaultencoding()
        self.failUnlessEqual(enc, encoding)
        self.failUnlessEqual(codecs.lookup(enc), codecs.lookup(encoding))


    def testSystemDefaultEncoding(self):
        """Test that component get system default encoding when appropriate.
        """
        comp = self.getComponent()
        self.failUnlessComponentEncodingEqual(comp, sys.getdefaultencoding())

    def testExplicitEncoding(self):
        """Test that component gets encoding from magic comment.
        """
        enc = random_encoding()
        comp = self.getComponent(head = "# encoding: %s\n" % enc)
        self.failUnlessComponentEncodingEqual(comp, enc)

    def testEncodingFromBOM(self):
        """Test that UTF-8 BOM results in utf-8 encoding
        """
        comp = self.getComponent(head = codecs.BOM_UTF8)
        self.failUnlessComponentEncodingEqual(comp, 'utf_8')

    def testBadEncodingFromBOM(self):
        """Test that UTF-8 BOM results in utf-8 encoding
        """
        expected = (SyntaxError,        # For bad module source
                    exception.Syntax,   # For bad file component source
                    exception.ConfigurationError # XXX: Bad module root?
                    )
        self.failUnlessRaises(expected,
                              self.getComponent,
                              head = codecs.BOM_UTF8 + "# encoding: ascii\n"
                              )

    def testEncodingFromFlagsEncoding(self):
        """Test that ``<%flags> encoding`` still works.
        """
        warnings.filterwarnings("ignore",
                                category=DeprecationWarning,
                                module=r'myghty\.compiler')
        enc = random_encoding()
        comp = self.getComponent("<%%flags> encoding = %s </%%flags>\n"
                                 % repr(enc))
        self.failUnlessComponentEncodingEqual(comp, enc)

    def x_testFlagsEncodingWarning(self):
        """Test that ``<%flags> encoding`` results in a warning.
        """
        resetwarnings()
        warnings.filterwarnings("error",
                                category=DeprecationWarning,
                                module=r'myghty\.compiler')
        self.failUnlessRaises(DeprecationWarning,
                              self.getComponent,
                              "<%flags> encoding='ascii' </%flags>\n")
        
    def testMagicCommentAndFlagsEncodingIsError(self):
        """Specifying a magic comment and an encoding flag is an error.

        Test that specifying a magic comment and a ``<%flags> encoding``
        is an error.
        """
        warnings.filterwarnings("ignore",
                                category=DeprecationWarning,
                                module=r'myghty\.compiler')
        self.failUnlessRaises(exception.Syntax,
                              self.getComponent,
                              head="# encoding: latin1\n"
                              "<%flags> encoding='ascii' </%flags>\n")

    def testMagicCommentInCompiledComponent(self):
        """Test that the magic comment get generated in compiled code.
        """
        encoding = random_encoding()
        compiled = self.getCompiled("# -*- encoding: %s -*-\n" % encoding)
        self.failUnlessEqual(parse_magic_comment(compiled), encoding)

    def testNoMagicCommentInCompiledComponent(self):
        """Test that the magic comment doesn't get generated if there's
        no magic comment in the source file.
        """
        compiled = self.getCompiled()
        self.failUnlessEqual(parse_magic_comment(compiled), None)


class MemoryComponentEncodingTests(FileComponentEncodingTests):

    def getComponent(self, head=""):
        return self.makeMemoryComponent(head + "\nbody\n")

    def _getCompiledFromComp(self, comp):
        buf = StringIO.StringIO()
        comp.component_source.get_object_code(self.interpreter.compiler.get(),
                                              buf)
        return buf.getvalue()

if __name__ == '__main__':
    runner = unittest.TextTestRunner(verbosity=2, descriptions=False)
    unittest.main(testRunner=runner)



import sys, os

from paste.util import import_string
import myghty.http.WSGIHandler as WSGIHandler
from myghty.resolver import *

from myghty.ext.routeresolver import RoutesResolver

def make_myghty_app(global_conf, 
                    package_name=None,
                    root_path=None,
                    **app_conf):
    if package_name:
        package = package_name
        if isinstance(package, (str, unicode)):
            package = import_string.simple_import(package+'.webconfig')
    
    config = package.config

    def myghtyapp(environ, start_response):
        return WSGIHandler.handle(environ, start_response,
                                  **config
                                  )
    return myghtyapp
    



"""Myghty server configuration.  should be neutral to HTTP environment, i.e. WSGI, standalone, etc.

This file should be executed in a context that includes "root" as the path to the application root."""


data_dir=os.path.join(root, 'cache')

resolver_strategy = [
    # request-level resolution: everything goes to the front controller
    ConditionalGroup(
            context="request", rules=[
                ResolveModule({r'(.*)' : 'zblog.controller.front:index'}),
                NotFound(),
            ]
    ),
    
    # front-controller resolution: front controller forwards requests into this realm
    # after performing security checks
    ConditionalGroup(
        context="frontcontroller", rules=[
            ResolvePathModule(os.path.join(root, 'lib/zblog/controller'), path_stringtokens=['index'], path_moduletokens=['index']),
            NotFound()
        ]
    ),
    
    # component/template resolution, for subrequests, component calls, inheritance
    
    # autohandlers
    ResolveUpwards(),
    
    # for the 'components' directory, we append '.myc' to incoming filenames so templates dont have to 
    # specify
    ResolveFile({'components':os.path.join(root, 'components')}, adjust=lambda u: re.sub(r'$', '.myc', u)),
    
    # page-level resolution; we convert directory names to index.myt
    PathTranslate((r'/$', '/index.myt')),
    ResolveFile({'htdocs':os.path.join(root, 'htdocs')}),
]

#debug_elements=['resolution']

attributes = {
    'config_file' : os.path.join(root, 'config/config.py'),
}

def preproc(source):
    """source pre-processor.  adds an import to the top of all components."""
    return """
<%global>
import zblog.domain.actions as actions    
</%global>
""" + source
python_pre_processor = preproc



# startup stuff.
import zblog
import zblog.database
zblog.load_config(attributes['config_file'])




from routes.util import redirect_to, url_for

class ApplicationController(object):
    def do_run_component(self, m, r, s, **params):
        m.global_args['url_for'] = url_for
        m.global_args['redirect_to'] = redirect_to


"""zblog general module.  Defines global application constants and helper functions."""

import os

"""application configuration.  the runtime environment should establish configuration details in this dictionary."""
config = {}

"""startup list.  when configuration is loaded, callables in this list will be executed."""
startup =[]

"""indicates if config file needs to be loaded."""
need_config = True


def load_config(file):
    """called by application starting point (such as a front controller or command line script) to initialize configuration with given filename."""
    global need_config
    if not need_config:
        return False
    if len(config) == 0:
        if os.access(file, os.F_OK):
            execfile(file, config)
            need_config=False
            for callable_ in startup:
                callable_()
    return need_config

def reset_config():
    """reset config - used by bootstrapper when an error occurs"""
    config.clear()
    global need_config
    need_config=True



# $Id: args.py 2013 2005-12-31 03:19:39Z zzzeek $
# args.py - component argument descriptor classes for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#


__all__ = ['ComponentArg', 'RequestArg', 'SubRequestArg', 'LocalArg', 'DynamicArg']

import myghty.exception as exception

class ComponentArg:
    def __init__(self, name, required = True, default = None, linenumber = None):
        self.name = name

        if default is None:
            self.required = required
        elif not default:
            self.default = None
            self.required = True
        else:
            self.default = default
            self.required = False
            
        self.linenumber = linenumber

    def do_get_arg(self, request, **params):
        raise NotImplementedError()

    def __repr__(self):
        return "%s(%s, required = %s)" % (
            self.__class__.__name__,
            repr(self.name),
            repr(self.required),
        )
        
    def get_arg(self, request, dictionary, **params):
        try:
            dictionary[self.name] = self.do_get_arg(request, **params)
        except KeyError:
            if self.required:            
                raise exception.MissingArgument("required %s argument %s not found" % (self.__class__.type, self.name))

class RequestArg(ComponentArg):
    type = "request"
    def do_get_arg(self, request, **params):
        return request.root_request_args[self.name]
        
class SubRequestArg(ComponentArg):
    type = "subrequest"
    def do_get_arg(self, request, **params):
        return request.request_args[self.name]
        
class LocalArg(ComponentArg):
    type = "component"
    def do_get_arg(self, request, **params):
        return params[self.name]
        
class DynamicArg(ComponentArg):
    type = "dynamic"
    def do_get_arg(self, request, **params):
        def find(request, params):
            yield params
            while request is not None:
                yield request.request_args
                request = request.parent_request
    
        for d in find(request, params):
            if d.has_key(self.name):
                return d[self.name]
        raise KeyError(self.name)



import testbase
import myghty.interp as interp
import myghty.component as component
import myghty.csource as csource
import myghty.resolver as resolver
import myghty.request as request
import myghty.exception as exception
import myghty.cache as cache
import unittest, sys, string, StringIO
import posixpath as unixpath

class ResolverStub:
    def __init__(self, result):
        self.result = result
        
    def resolve(self, uri, **params):
        return self.result


class ComponentRunner(testbase.MyghtyTest):
    
    def assert_exec(self, csource, args, output, interpreter = None):
        if interpreter is None:
            interpreter = interp.Interpreter()
        comp = interpreter.load_component(csource)
        buf = StringIO.StringIO()
        interpreter.execute(comp, request_args = args, out_buffer = buf)
        self.failUnlessEqual(buf.getvalue(), output)

    def assert_component(self, component, args, output, interpreter):
        buf = StringIO.StringIO()
        interpreter.execute(component, request_args = args, out_buffer = buf)
        self.assert_(buf.getvalue() == output)
        
class InheritTest(testbase.MyghtyTest):
    def setUp(self):
        self.create_file(self.components, 'autohandler', 'im an autohandler') 
    
    def tearDown(self):
        self.remove_file(self.components, 'autohandler')
            
    def testrecursiveparent(self):
        """test that a root autohandler component wont let itself be a parent of itself"""

        acs = csource.FileComponentSource("fooid", "main", "autohandler", unixpath.join(self.components, "autohandler"))
        
        i = interp.Interpreter(resolver = ResolverStub(resolver.Resolution(acs, None)))
        comp = i.load_component(acs)
        
        self.assert_(comp is not comp.parent_component, "test failed, returned recursive parent")
        self.assert_(comp.parent_component is None, "test failed, returned a parent component")

class AttrTest(ComponentRunner):
    def setUp(self):
        self.create_file(self.htdocs, "compa.myt", 
"""<%attr>
foo = 'fooval'
</%attr>
% m.call_next()""")
        self.create_file(self.htdocs, "compb.myt", 
"""<%flags>
inherit = '/compa.myt'
</%flags>\
<%attr>
bar = 'barval'
</%attr>
% m.call_next()""")
        self.create_file(self.htdocs, "compc.myt", 
"""<%flags>
inherit = '/compb.myt'
</%flags>\
<% m.base_component.attributes['bar'] %>
<% m.base_component.attributes['foo'] %>""")

    def testattrinherit(self):
        interpreter = interp.Interpreter(data_dir=self.cache, component_root=self.htdocs)
        self.assert_component('compc.myt', {}, "barval\nfooval", interpreter = interpreter)
        
    def tearDown(self):
        self.remove_file(self.htdocs, 'compa.myt')
        self.remove_file(self.htdocs, 'compb.myt')
        self.remove_file(self.htdocs, 'compc.myt')

class CacheTest(ComponentRunner):
    def setUp(self):
        self.create_file(self.htdocs, "memorytest.myt", """
<%flags>
    use_cache=True
    cache_type='memory'
</%flags>
im the memory test.
""")
        self.create_file(self.htdocs, "filetest.myt", """
<%flags>
    use_cache=True
    cache_type='file'
</%flags>
im the file test.
""")

    def testmemory(self):
        acs = csource.FileComponentSource("memid", "main", "memorytest.myt", unixpath.join(self.htdocs, "memorytest.myt"))
        interpreter = interp.Interpreter()
        self.assert_exec(acs, {}, "\nim the memory test.\n", interpreter = interpreter)
        self.assert_exec(acs, {}, "\nim the memory test.\n", interpreter = interpreter)
        comp = interpreter.code_cache['memid']
        self.assert_(interpreter.make_request(comp).get_cache(comp, type='memory')['_self'][0].getvalue() == "\nim the memory test.\n")

    def testfile(self):
        acs = csource.FileComponentSource("fileid", "main", "filetest.myt", unixpath.join(self.htdocs, "filetest.myt"))
        interpreter = interp.Interpreter(data_dir=self.cache)
        self.assert_exec(acs, {}, "\nim the file test.\n", interpreter = interpreter)
        self.assert_exec(acs, {}, "\nim the file test.\n", interpreter = interpreter)
        comp = interpreter.code_cache['fileid']
        self.assert_(interpreter.make_request(comp).get_cache(comp, type='file')['_self'][0].getvalue() == "\nim the file test.\n")
        
    def tearDown(self):
        self.remove_file(self.htdocs, 'memorytest.myt')
        self.remove_file(self.htdocs, 'filetest.myt')

################################################################
        
class MoreMemoryCacheTests(ComponentRunner):

    cache_type = 'memory'
    
    def setUp(self):
        # Basically we just need a component...
        self.create_file(self.htdocs, "cachetest.myt", "im the cache tester.")
        self.interp = interp.Interpreter(data_dir=self.cache)
        acs = csource.FileComponentSource("id", "main", "cachetest.myt",
                                          unixpath.join(self.htdocs,
                                                        "cachetest.myt"))
        self.comp = self.interp.load_component(acs)

    def tearDown(self):
        self.remove_file(self.htdocs, 'cachetest.myt')

    def get_cache(self):
        """Get a fresh Cache object for the component
        """
        return cache.Cache(component=self.comp, type=self.cache_type,
                           data_dir=self.cache)

    def cache_keys(self):
        """List the keys in our components cache.

        This lists the keys in the underlying NamespaceManager, not just
        those keys which are cached in the Cache object.
        """
        cache = self.get_cache()
        nm = cache.get_container(None).get_namespace_manager()
        # We introduced a bogus None key with the above get_container,
        # so filter it out...
        nm.acquire_read_lock()
        try:
            return [ x for x in nm.keys() if x is not None ]
        finally:
            nm.release_read_lock()

    def testClear(self):
        """Test that cache.clear() really works.

        We test that all entries in the underlying NamespaceManager actually
        get cleared.
        """
        cache = self.get_cache()
        cache.get_value('key1', createfunc=lambda: 'value1')
        self.failUnlessEqual(cache.dict.keys(), ['key1'])
        self.failUnlessEqual(self.cache_keys(), ['key1'])

        cache = self.get_cache()
        self.failUnlessEqual(cache.dict.keys(), [])
        self.failUnlessEqual(self.cache_keys(), ['key1'])

        cache.clear()
        self.failUnlessEqual(self.cache_keys(), [])

class MoreFileCacheTests(MoreMemoryCacheTests):
    cache_type = 'file'

class MoreDbmCacheTests(MoreMemoryCacheTests):
    cache_type = 'dbm'

class MoreMemcachedCacheTests(MoreMemoryCacheTests):
    cache_type = 'ext:memcached'

    def testClear(self):
        # FIXME:
        self.fail("MemcachedNamespaceManager.do_remove() is currently broken."
                  "  (It is a no-op.)")
    
    
################################################################

class myclass(component.ModuleComponent):
    def do_run_component(self, m, **params):
        m.write('hello world')

class ClassComponentTest(ComponentRunner):
    def testrun(self):

        acs = csource.ModuleComponentSource(arg = myclass)
        self.assert_exec(acs, {}, "hello world")

class FunctionComponentTest(ComponentRunner):
    def testmarg(self):
        def mymethod(m):
            m.write("hello world")
            
        acs = csource.ModuleComponentSource(arg = mymethod)
        self.assert_exec(acs, {}, "hello world")

    def testnoarg(self):
        def mymethod():
            request.instance().write("hello world")
            
        acs = csource.ModuleComponentSource(arg = mymethod)
        self.assert_exec(acs, {}, "hello world")

    def testmandarg(self):
        def mymethod(m, ARGS):
            m.write("hello world, args: " + repr(ARGS))
        
        acs = csource.ModuleComponentSource(arg = mymethod)
        self.assert_exec(acs, {'foo': 'bar'}, "hello world, args: " + repr({'foo': 'bar'}))

    def testcustomargs(self):
        def mymethod(x, y, z = 7):
            request.instance().write("hello world, args: %s %s %s" % (repr(x), repr(y), repr(z)))
        
        acs = csource.ModuleComponentSource(arg = mymethod)
        args = dict(x = 5, y = 2)
        self.assert_exec(acs, args, "hello world, args: 5 2 7")


class MethodComponentTest(ComponentRunner):
    def class_set_up(self):
        self.create_file(self.lib, 'mymodule3.py', 
"""
import myghty.component as component

def dostuff(m):
    m.write("this is dostuff")

def index(m):
    m.write("this is index")

class foo:
    def __init__(self):
        self.x = 7
    def __call__(self, m):
        m.write("this is foo __call__")
        
    def dofoo(self, m):
        m.write("this is foo dofoo(), x is " + str(self.x) )
        
bar = foo()

""")

        self.mymod2 = __import__('mymodule3')

    def class_tear_down(self):
        self.remove_file(self.lib, 'mymodule3.py')

    def testmodfunc(self):
        dostuff = self.mymod2.dostuff
        acs = csource.ModuleComponentSource(arg= dostuff)
        self.assert_exec(acs, {}, "this is dostuff")

    def testmodmethod(self):
        bar = self.mymod2.bar
        acs = csource.ModuleComponentSource(arg = bar.dofoo)
        self.assert_exec(acs, {}, "this is foo dofoo(), x is 7")

    def testmodcallable(self):
        acs = csource.ModuleComponentSource(arg = self.mymod2.bar)
        self.assert_exec(acs, {}, "this is foo __call__")
        
if __name__ == '__main__':
    runner = unittest.TextTestRunner(verbosity=2, descriptions=False)
    unittest.main(testRunner=runner)
                



import codecs, re, sys, unittest, warnings
from myghty import escapes

LATIN1_CHARS = ''.join(map(chr, range(256))).decode('latin1')

class EscapeTests(object):
    def escape(self, text):
        raise NotImplementedError

    def unescape(self, text):
        raise NotImplementedError

    TEST_CHARS = LATIN1_CHARS

    def testEscapedChars(self):
        escaped =  ''.join([char for char in self.TEST_CHARS
                            if self.escape(char) != char])
        should_escape = ''.join(re.findall(self.should_escape_re,
                                           self.TEST_CHARS))

        if escaped != should_escape:
            missing = ''.join([char for char in should_escape
                               if char not in escaped])
            shouldnt = ''.join([char for char in escaped
                                if char not in should_escape])
            self.failUnless(missing or shouldnt)
            if missing:
                self.fail("Should have escaped: %s" % repr(missing))
            if shouldnt:
                self.fail("Should not have escaped: %s" % repr(shouldnt))

    def testTakesUnicode(self):
        frappe = self.escape(u'frapp\u00C9')
        self.failUnless(isinstance(frappe, basestring))
        self.failUnlessEqual(frappe[:5], "frapp")

    def testTakesStr(self):
        hello = self.escape('hello')
        self.failUnless(isinstance(hello, basestring))
        self.failUnlessEqual(hello, "hello")

    def testStrMustBeAscii(self):
        if codecs.lookup(sys.getdefaultencoding()) != codecs.lookup('ascii'):
            warnings.warn("Skipping testStrMustBeAscii")
            return
        self.failUnlessRaises(UnicodeError, self.escape, '\xa0')

    def testUnescape(self):
        escaped = self.escape(self.TEST_CHARS)
        try:
            unescaped = self.unescape(escaped)
        except NotImplementedError:
            return
        unicode(unescaped)
        self.failUnlessEqual(unescaped, self.TEST_CHARS)
        
class html_escape_tests(EscapeTests, unittest.TestCase):
    escape = staticmethod(escapes.html_escape)
    should_escape_re = r'["&<>]'
    def testStrMustBeAscii(self): pass  # Is this right?

class xml_escape_tests(EscapeTests, unittest.TestCase):
    escape = staticmethod(escapes.xml_escape)
    should_escape_re = '["&\'<>]'
    def testStrMustBeAscii(self): pass  # Is this right?

class url_escape_tests(EscapeTests, unittest.TestCase):
    escape = staticmethod(escapes.url_escape)
    unescape = staticmethod(escapes.url_unescape)
    should_escape_re = r'[^-.\w]'

    def testPlus(self):
        self.failUnlessEqual(self.unescape('+'), ' ')
        self.failUnlessEqual(self.unescape('%2b'), '+')
        self.failUnlessEqual(self.escape(' '), '+')
        self.failUnlessEqual(self.escape('+'), '%2B')
        

class html_entities_escape_tests(EscapeTests, unittest.TestCase):
    escape = staticmethod(escapes.html_entities_escape)
    unescape = staticmethod(escapes.html_entities_unescape)

    should_escape_re = '["&<>\xa0-\xff]'

class htmlentityreplace_errors_tests(unittest.TestCase):
    def testEuro(self):
        self.failUnlessEqual(
            u'The cost was \u20ac12.'.encode('latin1', 'htmlentityreplace'),
            'The cost was &euro;12.'
            )
        
    def testNumeric(self):
        self.failUnlessEqual(u'\x81'.encode('ascii', 'htmlentityreplace'),
                             '&#x81;')

if __name__ == '__main__':
    unittest.main()



import zblog.controller
import zblog.database.mappers as mapper
from zblog.domain.user import User
import zblog.util.form as form
import StringIO, os
import zblog.domain.actions as actions
from zblog.controller import access_control

class LoginController(zblog.controller.Controller):
    def form(self):
        f = form.Form('login',[
            form.FormField('username', required=True),
            form.FormField('password', required=True),
        ])
        return f

    @access_control()
    def __call__(self, m, **kwargs):
        f = self.form()
        self.template(m, '/login.myt', form=f)
        
    @access_control(action=actions.Login())
    def login(self, m, r, ARGS, **kwargs):
        f = self.form()
        f.set_request(ARGS)

        u = User.mapper.get_by(name=f['username'].display)
        if u is None or not u.checkpw(f['password'].display):
            u = None
        if u is not None:
            s = m.get_session()
            s['user'] = u
            s.save()

            m.send_redirect("/", hard=True)
        else:
            f.append_error("Login failed")
            self.template(m, '/login.myt', form=f)
    
    @access_control(action=actions.Logout())
    def logout(self, m, **kwargs):
        s = m.get_session()
        if s.has_key('user'):
            del s['user']
            s.save()
            self.template(m, '/login.myt', form=self.form(), status="You have been logged out")
        else:
            self.template(m, '/login.myt', form=self.form(), status="You are not logged in")


    @access_control(login=False, action=actions.Register())
    def register(self, m, ARGS, validate=False):
        form = self.registerform(m)

        form.set_request(ARGS, validate=validate)
        
        if not validate:
            self.template(m, '/register.myt', form=form)
            return
            
        if form['password_set'].value and form['password_set'].value != form['password_repeat'].value:
            form.append_error("Passwords do not match")
            form.isvalid=False
            
        existing = User.mapper.get_by(name=form['name'].value)
        if existing is not None:
            form['name'].append_error("Username '%s' already exists" % form['name'].value)
            form.isvalid=False
            
        if not form.is_valid():
            self.template(m, '/register.myt', form=form)
            return

        mapper.begin()
        user = User()
        form.reflect_to(user)
        user.group=zblog.domain.user.user
        mapper.commit()
        form = self.form()
        form.append_success("Thanks for registering, %s!" % (user.name))
        self.template(m, '/login.myt', form=form)
    
            
    def registerform(self, m):
        f = form.Form('user',[
            form.IntFormField('user_id', attribute='id'),
            form.FormField('name', required=True),
            form.FormField('fullname', required=True),
            form.FormField('password_set', attribute='password', required=True),
            form.FormField('password_repeat', attribute=None, required=True)
        ])
        return f


index = LoginController()




import unittest, os
import testbase

def suite():
    modules_to_test = tuple(
        [ name
          for name, ext in map(os.path.splitext,
                               os.listdir(os.path.dirname(__file__)))
          if ext == '.py' and name.startswith('test_') ])
           
    alltests = unittest.TestSuite()
    for module in map(__import__, modules_to_test):
        alltests.addTest(unittest.findTestCases(module))
    return alltests

if __name__ == '__main__':
    testbase.runTests(suite())



import os, types, string

from myghty.resolver import ResolverRule
import myghty.csource as csource
import myghty.component as comp
from myghty.resolver import Resolution
import myghty.importer as importer

import routes
from routes.base import *

class RoutesComponentSource(csource.ModuleComponentSource):
    def __init__(self, objpath, module):
    	self.objpath = objpath
    	self.module = module
    	
    	arg = module
    	for t in objpath:
            arg = getattr(arg, t)
        
        name = "method:" + string.join(objpath, '_')
	    
        self.has_method = True
        last_modified = importer.mod_time(module)
        
        csource.ComponentSource.__init__(self, "module|%s:%s" % (module.__name__, name), last_modified = last_modified)
        
        self.module = module
        self.objpath = objpath
        self.name = name
        self.class_ = RoutesComponent
        self.callable_ = arg
    
    def reload(self, module):
        self.module = module        

        arg = module
        for t in self.objpath:
            arg = getattr(arg, t)
        self.callable_ = arg
	
    def can_compile(self):
        return False

class RoutesComponent(comp.FunctionComponent):
    def do_run_component(self, **params):
        m = params['m']
        r = params['r']
       
        # do stuff with routes config
        config = routes.request_config()
        config.redirect = m.send_redirect
        config.host = r.headers_in['host'].split(':')[0]
        if r.environ.get('HTTPS'):
            config.protocol = 'https'
        else:
            config.protocol = 'http'
 
        # update params with routes resolution args
        if hasattr(m, 'resolution'):
            params.update(m.resolution.override_args)
        
        # if we are linked to a method off of an object instance,
        # see if the object instance has a do_run_component method and run it
        if self.component_source.has_method:
            target = self.component_source.callable_.im_self
            if hasattr(target, 'do_run_component'):
                getattr(target, 'do_run_component')(**params)
                
        return comp.FunctionComponent.do_run_component(self, **params)
    
class RoutesResolver(ResolverRule):
    name = 'routeresolver'
    
    def _find_controllers(self, dirname, prefix=''):
        controllers = []
        for fname in os.listdir(dirname):
            filename = dirname + '/' + fname
            if os.path.isfile(filename) and fname.endswith('_controller.py') and not fname.startswith('application_'):
                controllers.append(prefix + fname)
            elif os.path.isdir(filename):
                controllers.extend(self._find_controllers(filename, prefix=fname+'/'))
        return controllers
    
    def _get_controllers(self):
        clist = {}
        controllers = self._find_controllers(self.controller_root)
        for con in controllers:
            key = con[:-14] # Remove _controller.py
            clist[key] = self.controller_root + '/' + con
        return clist
    
    def __init__(self, mapper=None, controller_root=None, **params):
        self.mapper = mapper
        self.controller_root = controller_root
    
    def do_init_resolver(self, resolver, remaining_rules, **params):
        self.clist = self._get_controllers()
        self.mapper.create_regs(self.clist.keys())
    
    def do(self, uri, remaining, resolution_detail, **params):
        if resolution_detail is not None: resolution_detail.append("resolverouteresolver:" + uri)

        if os.environ['MYGHTY_ENV'] == 'development':
            self.clist = self._get_controllers()
            self.mapper.create_regs(self.clist.keys())
        
        match = self.mapper.match(uri)
        
        if match:
            controller = match['controller']
            action = match['action']
            if action.startswith('_'):
                return remaining.next().do(uri, remaining, resolution_detail, **params)
            
            # Load up the request local singleton config
            config = routes.request_config()
            config.mapper = self.mapper
            config.mapper_dict = match.copy()

            # Remove the action/controller, rest of the args pass to the function
            del match['action']
            del match['controller']

            filename = self.clist[controller]
            classname = controller.split('/')[-1].lower()
            module = importer.filemodule(filename)
            resolution_detail.append("\nController:%s, Action:%s" % (controller, action))
            cs = RoutesComponentSource(
                module=module,
                objpath=[classname, action],
                )
            #raise repr(cs.__dict__)
            return Resolution(cs, resolution_detail, override_args = match)
        else:
            return remaining.next().do(uri, remaining, resolution_detail, **params)



import unittest, os, sys, StringIO, itertools, inspect, re, tempfile, time

from myghty import util, interp, exception, csource, importer
from myghty.component import Component


class MyghtyTest(unittest.TestCase):
    def __init__(self, *args, **params):
        unittest.TestCase.__init__(self, *args, **params)
        
        # make ourselves a Myghty environment
        self.root = os.path.abspath(os.path.join(os.getcwd(),
                                                 'testroot',
                                                 self.__class__.__module__,
                                                 self.__class__.__name__))
        
        # some templates
        self.htdocs = os.path.join(self.root, 'htdocs')
        
        # some more templates
        self.components = os.path.join(self.root, 'components')
        
        # data dir for cache, sessions, compiled
        self.cache = os.path.join(self.root, 'cache')
        
        # lib dir for some module components
        self.lib = os.path.join(self.root, 'lib')
        sys.path.insert(0, self.lib)
        
        for path in (self.htdocs, self.components, self.cache, self.lib):
            util.verify_directory(path)
        
        self.class_set_up()

    def class_set_up(self):
        pass

    def class_tear_down(self):
        pass
        
    def __del__(self):
        self.class_tear_down()
        
    def create_file(self, dir, name, contents):
        file = os.path.join(dir, name)
        f = open(file, 'w')
        f.write(contents)
        f.close()
        
    def create_directory(self, dir, path):
        util.verify_directory(os.path.join(dir, path))
        
    def remove_file(self, dir, name):
        if os.access(os.path.join(dir, name), os.F_OK):
            os.remove(os.path.join(dir, name))


def deindent(text):
    """De-indent docstring like text.
    """
    # Hack use inspect.getdoc() to do the de-indentation
    return inspect.getdoc(property(doc=text))
    
class ComponentTester(MyghtyTest):

    config = {}

    srcFiles = {}
    
    def __init__(self, *args, **params):
        MyghtyTest.__init__(self, *args, **params)

        # Clean up from previous runs
        now = time.time()
        for root, dirs, files in os.walk(self.root):
            try:
                for fn in [ os.path.join(root, f) for f in files ]:
                    if os.stat(fn).st_mtime < now - 60:
                        os.unlink(fn)
            except IOError:
                pass
            
    def class_set_up(self):
        MyghtyTest.class_set_up(self)
        for name, content in self.srcFiles.items():
            self.createSrcFile(name, content)

    def createSrcFile(self, filename, content):
        m = re.match(r'\A ( htdocs | lib | components ) /+', filename, re.X)
        if m:
            filename = filename[m.end():]
            srcdir = getattr(self, m.group(1))
        else:
            srcdir = self.htdocs
        self.create_file(srcdir, filename, deindent(content))

    def makeInterpreter(self):
        config = dict(component_root=self.htdocs,
                      raise_error=True,
                      data_dir=self.cache
                      )
        config.update(self.config)
        return interp.Interpreter(**config)

    def setUp(self):
        self.interpreter = self.makeInterpreter()
        self.outputBuffer = StringIO.StringIO()

    output = property(lambda self: self.outputBuffer.getvalue())

    tidyoutput = property(lambda self: re.sub(r'\s+', ' ',
                                              self.output.strip()))

    def create_anonymous_file(self, src, dir=None, suffix='.myt'):
        if dir is None:
            dir = os.path.join(self.root, 'tmp')
        util.verify_directory(dir)
        fd, path = tempfile.mkstemp(prefix='anon', suffix=suffix, dir=dir,
                                    text=True)
        os.write(fd, deindent(src))
        os.close(fd)
        return os.path.basename(path)

    def makeModule(self, src):
        # create a python module in self.lib
        name = self.create_anonymous_file(src, dir=self.lib, suffix='.py')
        return importer.module(os.path.splitext(name)[0])
        
    def makeModuleComponent(self, arg):
        # src can be a string like "module:callable", etc...
        # or it can be an actually callable, etc...
        return self.interpreter.load_module_component(arg=arg)
        
    def makeFileBasedComponent(self, src):
        name = self.create_anonymous_file(src, dir=self.htdocs)
        return self.loadComponent("/%s" % name)

    def makeMemoryComponent(self, src):
        return self.interpreter.make_component(deindent(src))

    def makeComponent(self, src):
        """Make a component of the default sort.

        The idea is that this can be specialized by subclasses to change
        the type of component that gets constructed by code like:

             self.runComponent('''# Here's source code for some component
                               foo
                               ''')
        """                       
        return self.makeFileBasedComponent(src)
    
    def loadComponent(self, component):
        if not isinstance(component, basestring):
            # Wrap callable, etc... to a module component
            return self.makeModuleComponent(component)

        if re.search(r'\s', component):
            # if string contains white-space, interpret it as the
            # source for a myghty component, rather than the path
            # to be resolved.
            return self.makeComponent(component)

        return self.interpreter.load(component)

    def runComponent(self, component, config={}, **request_args):
        request_config = dict(request_args=request_args,
                              out_buffer=self.outputBuffer
                              )
        request_config.update(config)

        if not isinstance(component, Component):
            component = self.loadComponent(component)

        return self.interpreter.execute(component, **request_config)

    def failUnlessRaisesWrappedError(self, wrapped_exception,
                                     func, *args, **kw):
        def _unwrap_error(func, args, kw):
            try:
                func(*args, **kw)
            except exception.Error, error:
                if error.wrapped is None:
                    raise
                raise error.wrapped
            except:
                self.fail("expected myghty Error, got %s"
                          % sys.exc_info()[1])
        self.failUnlessRaises(wrapped_exception, _unwrap_error, func, args, kw)


# Add advice to unittest.TestResult.addError to unwrap wrapped Errors
TestResult_addError = unittest.TestResult.addError
instancemethod = type(TestResult_addError)
def addError(self, test, err):
    if isinstance(err[1], exception.Error):
        err = getattr(err[1], 'raw_excinfo', err)
    TestResult_addError.im_func(self, test, err)
unittest.TestResult.addError = addError


def runTests(suite):
    runner = unittest.TextTestRunner(verbosity = 2, descriptions =1)
    runner.run(suite)

if __name__ == '__main__':
    unittest.main()
    



# $Id: importer.py 2013 2005-12-31 03:19:39Z zzzeek $
# importer.py - Myghty memory-managed module importer
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#

import string, os, sys, imp, re, stat, types, time, weakref, __builtin__

"""
module loading and management.  loads modules by file paths directly, as well
as via module names.  keeps track of last modified time to provide a "reasonable"
level of "reload when changed" behavior without restarting the application.  By 
"reasonable" we include the module itself, but not its parent package or any 
of its named dependencies.

in the case of file-based modules, which correspond to compiled templates as well
as module components resolved via file paths, they are kept out
of sys.modules so they can be cleanly reloaded when modified, and removed from 
memory when they fall out of scope.   To maintain "importability" of these 
modules, the builtin __import__ method is overridden application-wide to 
search for these modules in a local weak dictionary first before proceeding to 
normal import behavior.

in the case of modules located by a package/module name, 
they are loaded into sys.modules via the default __import__ method
and are reloaded via reload().  For these modules, the "singleton" behavior of 
Python's regular module system applies.  This behavior includes the caveats that old 
attributes stay lying around, and the module is reloaded "in place" which in rare 
circumstances could affect code executing against the module.  The advantage is that
the module's parent packages all remain pointing to the correctly reloaded module 
and no exotic synchronization-intensive "reconnection" of newly reloaded modules 
to their packages needs to happen.

The "importability" of a module loaded here is usually not even an issue as it
typcially is only providing Myghty components which are solely invoked by the Interpreter.
However, in addition to the case where the developer is explicitly importing 
from a module that also provides Myghty components, the other case when the module requires 
import is when a class defined within it is deserialized, such as from a cache or session 
object; hence the need to override __import__ as well as maintaining the structure
of packages.  

"""

modules = weakref.WeakValueDictionary()

# override __import__ to look in our own local module dict first
builtin_importer = __builtin__.__import__
def import_module(name, globals = None, locals = None, fromlist = None):
    try:
        return modules[name].module
    except KeyError:
        return builtin_importer(name, globals, locals, fromlist)

__builtin__.__import__ = import_module

class ModValue:
    """2.3 is not letting us make a weakref to a module.  so create a lovely 
    circular reference thingy and weakref to that."""
    def __init__(self, module):
        self.module = module
        module.__modvalue__ = self

def module(name):
    """imports a module by string name via normal module importing, attaches timestamp information"""
    if name == '__main__':
        return sys.modules[name]
    mod = builtin_importer(name)
    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
        
    if not hasattr(mod, "__modified_time"):
        mod.__modified_time = modulemodtime(mod)
        
    mod.__is_file = False
    return mod

def filemodule(path, id = None, reload = True, forcereload = False):
    """loads a module directly from a file path."""
    
    if id is None:
        id = re.sub(r'\W+','_',path)

    if not forcereload:
        try:
            module = modules[id].module
            if not reload or module.__modified_time >= modulemodtime(module):
                return module
        except KeyError:
            pass
        
    modfile = open(path, 'r')
    try:
        #print "loading: " + path

        # Check mtime before loading module, so that modified_time
        # is guaranteed not to be later than the mtime of the loaded
        # version of the file.
        modified_time = os.fstat(modfile.fileno())[stat.ST_MTIME]
        module = imp.load_source(id, path, modfile)
        del sys.modules[id]
        modules[id] = ModValue(module)
        module.__modified_time = modified_time
        module.__is_file = True
        return module
    finally:
        modfile.close()


def reload_module(module):
    """reloads any module that was loaded with filemodule(), if its 
    modification time has changed.
    """    
    
    if not hasattr(module, '__modified_time'):
        # if we didnt load it, we dont change it
        return module
    elif module.__modified_time < modulemodtime(module):
        if module.__is_file is False:
            #print "regular reload: " + module.__name__
            # Get mtime before reload to ensure it is <= the actual mtime
            # of the reloaded module.
            modified_time = modulemodtime(module)
            reload(module)
            module.__modified_time = modified_time
            return module
        else:
            file = module.__file__
            file = re.sub(r'\.pyc$|\.pyo$', '.py', file)
            return filemodule(file, id = module.__name__, forcereload = True)
    else:
        return module
        

def mod_time(module):
    try:
        return module.__modified_time
    except AttributeError:
        return modulemodtime(module)

def modulemodtime(module):
    """returns the modified time of a module's source file"""
    try:
        file = module.__file__
        pyfile = re.sub(r'\.pyc$|\.pyo$', '.py', file)
        if os.access(pyfile, os.F_OK): 
            file = pyfile

        #print "checking time on " + file
        st = os.stat(file)
        return st[stat.ST_MTIME]
    except AttributeError:
        return None

    

class ObjectPathIterator:
    """walks a file path looking for a python module.  once it loads the
    python module, then continues walking the path into module's attributes."""
    
    def __init__(self, path, reload = True):
        self.path = path
        self.reload = reload
        self.module = None
        self.objpath = []
        if isinstance(path, types.ModuleType):
            self.module = path
            if reload:
                reload_module(self.module)
            self.last_modified = None
            
    def get_unit(self, tokens, stringtokens = [], moduletokens = []):
        if isinstance(self.path, str):
            return self.get_string_unit(tokens + stringtokens)
        else:
            return self.get_attr_unit(tokens + moduletokens)
        
    def get_string_unit(self, tokens):
        for token in tokens:
            path = self.path + "/" + token

            #print "check path " + repr(path)
            if self._check_module(path):
                return (self.path, token)
                
            if not os.access(path, os.F_OK):
                continue

            self.path = path
            return (self.path, token)
        else:
            raise StopIteration
            
    def get_attr_unit(self, tokens):
        for token in tokens:
            try:
                #print "check attr path " + repr(self.path) + " " + token
                attr = getattr(self.path, token)
                if isinstance(attr, types.ModuleType):
                    raise AttributeError(token)
                self.path = attr
                self.objpath.append(token)
                return (self.path, token)
            except AttributeError:
                continue
        else:
            self.path = None
            raise StopIteration
            
    def _check_module(self, path):
        try:
            st = os.stat(path + ".py")
        except OSError:
            return False

        if stat.S_ISREG(st[stat.ST_MODE]):
            self.path = filemodule(path + ".py", reload = self.reload)
            self.module = self.path
            self.last_modified = mod_time(self.module)
            return True



"""a simple access-control ruleset.  Each class defines a different kind of user "action",
along with a function containing all necessary logic to determine if the action is allowed
for the currently logged in user (if any).  These objects can be used both to disable/hide
links and buttons, as well as when the message comes into a controller to determine whether
to perform the action or produce a 403 Forbidden exception."""
import re, string
import zblog
from zblog.domain.blog import *


actions = {}
class ActionSingleton(type):
    def __call__(self):
        try:
            name = string.lower(re.sub(r"(\w)([A-Z])", r"\1_\2", self.__name__))
            return actions[name]
        except KeyError:
            return actions.setdefault(name, type.__call__(self, name))
    
class Action(object):
    __metaclass__ = ActionSingleton
    def __init__(self, name):
        self.name = name
        
    def access(self, user, **kwargs):
        return False

class Bootstrap(Action):
    """defines the initial config file creation and database creation.  only allowed
    if the application has no configuration defined."""
    def access(self, user, **kwargs):
        return zblog.need_config 
    
class Login(Action):
    def access(self, user, **kwargs):
        return True

class Logout(Action):
    def access(self, user, **kwargs):
        return True

class Manage(Action):
    def access(self, user, **kwargs):
        return user is not None and user.is_administrator()
        
class AdminUsers(Action):
    def access(self, user, **kwargs):
        return user is not None and user.is_administrator()

class LoggedIn(Action):
    def access(self, user, **kwargs):
        return user is not None
                
class EditBlog(Action):
    def access(self, user, blog_id=None, **kwargs):
        if user is None:
            return False
        if blog_id and not user.is_administrator():
            blog = Blog.mapper.get(blog_id)
            if blog is None:
                return False
            return user.id==blog.owner.id
        else:
            return user.is_administrator()

class CreatePost(Action):
    def access(self, user, blog_id=None, blog=None, **kwargs):
        if user is None:
            return False
        if blog is None:
            blog = Blog.mapper.get(blog_id)
            if blog is None:
                return False
        return blog.owner.id==user.id or user.is_administrator()

class EditPost(Action):
    def access(self, user, post_id=None, post=None, **kwargs):
        if user is None:
            return False
        if post is None:
            post = Post.mapper.get(post_id)
            if post is None:
                return False
        return post.user.id==user.id or user.is_administrator() or user.id==post.blog.owner.id

class Register(Action):
    def access(self, user, **kwargs):
        return user is None
    
class CreateComment(Action):
    def access(self, user, **kwargs):
        return user is not None
        
class DeleteComment(Action):
    pass





from shoppingmodel import *


store = Category('store', 'The Myghty Store', [])

    
tshirts = Category("tshirts", "T Shirts",
    [
        Item("Myghty T Shirt", "A lovely T-shirt made with genuine Microsoft Paint, featuring the Myghty logo", "tshirt_myghty.gif", None, variants =
            [
                Variant('size', 'small', 9.99),
                Variant('size', 'medium', 10.99),
                Variant('size', 'large', 11.99),
                Variant('size', 'xtra large', 12.99),
                Variant('color', 'white'),
                Variant('color', 'black'),
                Variant('color', 'blue')
            ]
        ),
        
        Item("Python T Shirt", "A lovely T-shirt, carefully line drawn in less than five minutes, featuring the Python logo", "tshirt_python.gif", 10.99, variants = 
            [
                Variant('size', 'small'),
                Variant('size', 'medium'),
                Variant('size', 'large'),
                Variant('size', 'xtra large'),
            ]
        ),
        Item("Snake T Shirt", "Our finest shirt, features more pixels and colors than our other shirts.", "tshirt_snake.gif", 10.99, variants = 
            [
                Variant('size', 'small'),
                Variant('size', 'medium'),
                Variant('size', 'large'),
                Variant('size', 'xtra large'),
            ]
        )
    ],
    store)
    
mugs = Category("mugs", "Mugs",
    [
        Item("Myghty Mug", "Get your caffeine on with this mediocre line drawing", "mug_myghty.gif", 5.99),
        Item("Python Mug", "Get your caffeine on with this mediocre line drawing", "mug_python.gif", 5.99, 
                [
                Variant('color', 'white'),
                Variant('color', 'black'),
                Variant('color', 'blue')
                ]
        ),
        Item("Snake Mug", "Get your caffeine on with this mediocre line drawing", "mug_snake.gif", 5.99),
    ],
    store)
    

hats = Category("hats", "Hats",
    [
        Item("Myghty Hat", "This hat I could barely even draw, but you'll look cool.", "hat_myghty.gif", 8.99,
            [
                Variant('size', 'small'),
                Variant('size', 'medium'),
                Variant('size', 'large'),
            ]),
        Item("Python Hat", "This hat I could barely even draw, but you'll look cool.", "hat_python.gif", 6.75),
        Item("Snake Hat", "This hat I could barely even draw, but you'll look cool.", "hat_snake.gif", 6.75),
    ],
    store)


featureditems = Category("featureditems", "Featured Items", [
        hats.items[2],
        mugs.items[1],
        tshirts.items[0]
    ],
    None)
    



"""shopping cart runner."""

import os, sys, re

[sys.path.insert(0, path) for path in ['../../lib', '../../doc/lib', './lib']]

import myghty.http.HTTPServerHandler as HTTPServerHandler

# determine port number
try:
    port = int(sys.argv[1])
except:
    port = 8000

# create local cache directory to store generated files + sessions
if not os.access('./cache', os.F_OK):
    os.mkdir('./cache')


# now set up standalone server    
httpd = HTTPServerHandler.HTTPServer(

    # port num
    port = port,

    handlers = [
    
        # serve all URIs that do not start with /docs/ with an HTTPHandler.
        {r'(?!/docs/)' : HTTPServerHandler.HSHandler(
            data_dir = './cache',

            # resolve URIs in the 'shoppingcontroller' module first.
            # here, we do it directly by path.  But it can also be
            # broken down into regular expressions for each component
            # via the 'module_components' parameter.
            module_root = ['shoppingcontroller'],

            # else, resolve URIs to one of three component roots.
            component_root = [            
                        {'store_comp':'./components'},
                        {'store_templ':'./templates'},
                        {'store_htdocs':'./htdocs'},
                    ],

            # interpreter attributes, custom config settings used by the 
            # shopping cart controller.
            attributes = {
                'store_uri' : '/',
                'store_document_uri' : '/docs/',
                'store_path' : '/',
            },
            
            )
        
        },

        # docroots, used to determine r.filename, and 
        # also will be served directly if Interpreter rules did not match.
        {r'/source/(.*)' : './'},
        {r'/docs/(.*)' : './htdocs'},
    ]
)
    
print "HTTPServer listening on port %d" % httpd.port

httpd.serve_forever()




# $Id: HTTPServerHandler.py 2133 2006-09-06 18:52:56Z dairiki $
# HTTPServerHandler.py - standalone HTTP server for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#

import myghty.interp
import myghty.request
import myghty.resolver
import myghty.exception as exception
from myghty.buffer import LinePrinter
import myghty.http.HTTPHandler as HTTPHandler
import BaseHTTPServer, SocketServer
from myghty.util import *
import os, sys, cgi, re, types, traceback
import mimetypes,posixpath,shutil,urllib


class HTTPServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
    """main server class.  just overrides the basic webserver."""
    def __init__(self, port = 8000, docroot = None, handlers = None, text_only = True, **params):
        self.params = params
        self.port = port
        self.text_only = text_only
        self.daemon_threads = False
        server_address = ('', port)

        
        if handlers is None:
            handlers = [{r'(.*)':HSHandler(**params)}]

        if docroot is not None:
            if not (isinstance(docroot, types.ListType)):
                handlers.append(docroot)
            else:
                handlers += docroot
            
        self.handlers = OrderedDict(handlers)
            
        BaseHTTPServer.HTTPServer.__init__(self, server_address, HTTPRequestHandler)
        

class HTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    """request handling for the webserver.  this is the engine of the standalone server."""
    def do_POST(self):
        self.handle_request()
    
    def do_GET(self):
        self.handle_request()

    def handle_request(self):
        httpreq = HSHTTPRequest(self)
    
        try:    
            if (
                self.server.text_only and httpreq.content_type[0:4] != "text"
            ):
                self.serve_file(httpreq)
            else:
                for key in self.server.handlers.keys():
                    match = re.match(key, httpreq.path_info)
                    if match:
                        handler = self.server.handlers[key]
                        if match.lastindex >= 1:
                            httpreq.converted_path_info = match.group(1)
                        else:
                            httpreq.converted_path_info = httpreq.path_info
                            
                        if type(handler) == str:
                            self.serve_file(httpreq)
                        else:
                            handler.handle(httpreq)
                        return
        except:
            traceback.print_exc(file=sys.stdout)
            self.send_error(500, "Server Error")
                

    # methods ripped from SimpleHTTPServer to handle sending binary
    # files, non-myghty files
    def translate_path(self, path):
        """Translate a /-separated PATH to the local filename syntax.

        Components that mean special things to the local file system
        (e.g. drive or directory names) are ignored.  (XXX They should
        probably be diagnosed.)

        """
        
        for key in self.server.handlers.keys():
            match = re.match(key, path)
            if match:
                root = self.server.handlers[key]
                if type(root) != str:
                    continue
                if match.lastindex >=1:
                    path = match.group(1)
                break
        else:
            return None

        path = posixpath.normpath(urllib.unquote(path))
        words = path.split('/')
        words = filter(None, words)
        path = root
        
        for word in words:
            drive, word = posixpath.splitdrive(word)
            head, word = posixpath.split(word)
            if word in (os.curdir, os.pardir): continue
            path = posixpath.join(path, word)
        return path

    def guess_type(self, path):
        """Guess the type of a file.

        Argument is a PATH (a filename).

        Return value is a string of the form type/subtype,
        usable for a MIME Content-type header.

        The default implementation looks the file's extension
        up in the table self.extensions_map, using text/plain
        as a default; however it would be permissible (if
        slow) to look inside the data to make a better guess.

        """

        base, ext = posixpath.splitext(path)
        if ext in self.extensions_map:
            return self.extensions_map[ext]
        ext = ext.lower()
        if ext in self.extensions_map:
            return self.extensions_map[ext]
        else:
            return self.extensions_map['']

    extensions_map = mimetypes.types_map.copy()
    extensions_map.update({
    '': 'text/html', # Default
    '.myt':'text/html',
    '.myc':'text/html',
    '.py': 'text/plain',
    '.c': 'text/plain',
    '.h': 'text/plain',
    })

    
    def serve_file(self, httpreq):
        ctype = httpreq.content_type
        if ctype.startswith('text/'):
            mode = 'r'
        else:
            mode = 'rb'
        
        path = httpreq.filename
        if path is None:
            self.send_error(404, "File not found")
            return
        try:
            f = open(path, mode)
        except IOError:
            self.send_error(404, "File not found")
            return
        self.send_response(200)
        self.send_header("Content-type", ctype)
        self.send_header("Content-Length", str(os.fstat(f.fileno())[6]))
        self.end_headers()
        shutil.copyfileobj(f, self.wfile)
        f.close()
        
# now, the myghty stuff.        

def handle(httpreq, interpreter_name = None, **params):
    return HTTPHandler.handle_http(HSHandler, interpreter_name = interpreter_name, httpreq = httpreq, **params)


class HSHandler(HTTPHandler.HTTPHandler):
    def __init__(self, **params):
        HTTPHandler.HTTPHandler.__init__(self, None, LinePrinter(sys.stderr), **params)

    def do_get_init_params(self, httpreq, **params):
        return params
    
    def do_get_resolver(self, **params):
        return HSResolver(**params)
    
    def do_make_request_impl(self, httpreq, **params):
        return HSRequestImpl(httpreq, **params)
        
    def do_get_component(self, httpreq, **params):
        return httpreq.converted_path_info
        
    def do_handle_result(self, httpreq, status, message):
        # in the case that we had an error, do a send_error, which shows the 
        # BaseHTTPServer's error page
        # for codes below 300, do nothing;  this will send no content, unless
        # the application has (as it does for a redirect)
        if status >= 400:
            if message is not None:
                httpreq.handler.send_error(status, message)
            else:
                httpreq.handler.send_error(status)

    def handle(self, httpreq):
        HTTPHandler.HTTPHandler.handle(self, httpreq)


class HSHTTPRequest(HTTPHandler.HTTPRequest):
    def __init__(self, httpreqhandler):
        HTTPHandler.HTTPRequest.__init__(self)

        self.handler = httpreqhandler
        headers = httpreqhandler.headers
        
        for key in headers.keys():
            self.headers_in.add(key, headers[key])
                        
        self.method = httpreqhandler.command
        
        match = re.match(r"([^\?]*)(?:\?(.*))?", httpreqhandler.path)
        if match:
            self.path_info = match.group(1)
            if match.lastindex >=2:
                self.args = match.group(2)
            else:
                self.args = None
        else:
            self.path_info = None
            self.args = None

        self.content_type = self.handler.guess_type(self.path_info)
        self.filename = self.handler.translate_path(self.path_info)
        
        # cobble together a cgi.FieldStorage object from what we have
        
        cgienviron = {
                'QUERY_STRING': self.args,
                'REQUEST_METHOD': self.method,
        }
        
        if self.headers_in.has_key('content-type'):
            cgienviron['CONTENT_TYPE'] = self.headers_in['content-type']
        if self.headers_in.has_key('content-length'):
            cgienviron['CONTENT_LENGTH'] = self.headers_in['content-length']
            
        self.fieldstorage = cgi.FieldStorage(
            fp = self.handler.rfile,
            environ = cgienviron,
            keep_blank_values = True
        )

    def do_send_headers(self):
        self.handler.send_response(self.status)
        headers = self.get_response_headers()
        for key, value in headers.iteritems():
            self.handler.send_header(key, value)

        self.handler.end_headers()

class HSResolver(HTTPHandler.HTTPResolver): pass


class HSWriter(HTTPHandler.HTTPWriter):pass
        

class HSRequestImpl(HTTPHandler.HTTPRequestImpl):
    def __init__(self, httpreq, **params):
        HTTPHandler.HTTPRequestImpl.__init__(self, httpreq, LinePrinter(sys.stderr), LinePrinter(sys.stderr), **params)
        
    def do_make_request_args(self, httpreq, **params):
        return self.request_args_from_fieldstorage(httpreq.fieldstorage)

    def do_get_out_buffer(self, httpreq, out_buffer = None, **params):
        if out_buffer is None:
            return HSWriter(httpreq, httpreq.handler.wfile, **params)
        else:
            return out_buffer



if __name__ == '__main__':
    params = {}
    for item in sys.argv[1:]:
        (key, value) = item.split('=', 1)
        params[key] = eval(value)

    httpd = HTTPServer(**params)
    print "HTTPServer listening on port %d" % httpd.port
    httpd.serve_forever()
 



# $Id: HTTPHandler.py 2028 2006-01-16 23:39:04Z zzzeek $
# generic HTTP handler library for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#

import myghty.buffer
import types, string, re, sys
import myghty.resolver
import myghty.interp
from myghty.util import *
import myghty.session as session
import myghty.exception as exception

__all__ = ['HTTPHandler', 'HeaderTable', 'HTTPRequest', 'HTTPWriter', 'HTTPResolver', 'handle_http']

handlers = {}

def handle_http(handlerclass, interpreter_name = None, interp = None, request_impl = None, component = None, **params):
    """handles an HTTP request using the named HTTPHandler instance.  if an instance of the name does not
    exist, it is created using the given configuration parameters.  
    
    implementing modules should define their own handle() method that calls this one
    with the appropriate arguments."""
    return get_handler(handlerclass, interpreter_name, **params).handle(interp = interp, request_impl = request_impl, component = component, **params)

def get_handler(handlerclass, interpreter_name = None, **params):
    """returns an HTTPHandler instance keyed off the given name.  if the named instance does not exist,
    it is created using the given configuration parameters."""
    interpreter_name = handlerclass.__name__ + "_" + (interpreter_name or 'main')
    try:
        handler = handlers[interpreter_name]
    except KeyError:
        handler = handlerclass(**params)
        handlers[interpreter_name] = handler

    return handler

class HTTPHandler:
    """entry point for HTTP requests, configures and maintains a reference to 
    an instance of a Myghty intepreter, and
    serves as a factory for many of the
    other implementing objects in this module.   override the do_*** methods of 
    this class, and call the handle() method inside the
    the request-handling point of the implementing environment."""
    
    def __init__(self, httpreq, logbuffer, **params):
        self.params = self._init_params(httpreq, logbuffer, **params)
        self.interp = myghty.interp.Interpreter(**self.params)

    def _init_params(self, httpreq, logbuffer, **params):
        params = self.do_get_init_params(httpreq, **params)

        if not params.has_key('allow_globals'):
            params['allow_globals'] = []

        params['allow_globals'].append('r')

        if params.setdefault('use_session', False):
            params['allow_globals'].append('s')

        debug_elements = params.setdefault('debug_elements', [])

        # deprecated
        if params.setdefault('log_component_loading', False):
            debug_elements.append('classloading')

        # deprecated
        if params.setdefault('log_cache_operations', False):
            debug_elements.append('cache')

        if len(debug_elements) > 0 and not params.has_key('debug_file'):
            params['debug_file'] = logbuffer
            
        if not params.has_key('resolver'):
            d = dict(params)
            if d.has_key('debug_file'): del d['debug_file']
            params['resolver'] = self.do_get_resolver(debug_file = None, **d)

        if not params.has_key('data_dir'):
            raise exception.ConfigurationError("data_dir config is required")

        if not params.has_key('component_root') and not params.has_key('module_components') and not params.has_key('resolver_strategy') and not params.has_key('module_root'):
            raise exception.ConfigurationError("No file resolution rules detected - component_root, module_components, and/or resolver_strategy")

        return params
    
    def do_get_init_params(self, httpreq, **params):raise NotImplementedError()
    
    def do_get_resolver(self, **params):raise NotImplementedError()
    
    def do_make_request_impl(self, httpreq, **params):raise NotImplementedError()
    
    def do_get_component(self, httpreq, **params):raise NotImplementedError()
    
    def do_handle_result(self, httpreq, status_code, reason):
        """called at the end of the request.  in a normal response, headers and probably
        content have already been sent.  in the case of errors or redirects, this method
        should insure that headers have been properly sent as required by the environment.
        in all cases, appropriate return data should be returned from this method which will 
        be returned to the calling environment."""
        raise NotImplementedError()
    
    def handle(self, httpreq, interp = None, request_impl = None, component = None, **params):
        """handles an HTTP request.  implementing classes may want to override this
        to provide environment-specific request arguments that are then translated into the implementing
        module's own HTTPRequest implementation.

        httpreq - HTTPRequest request object or compatible interface
        interp - optional Interpreter object to override the default
        request_impl - optional RequestImpl object the default
        component - optional Component object or request path to serve
        **params - configuration parameters which override the default parameters
        """

        if interp is None:
            interp = self.interp
    
        implparams  = self.params.copy()
        implparams.update(params)
    
        if not request_impl:
            request_impl = self.do_make_request_impl(httpreq, **implparams)
            
        if not component:
            if implparams.has_key('component'):
                component = implparams['component']
            else:
                component = self.do_get_component(httpreq, **implparams)
                
        reason = None
        request = interp.make_request(component = component, request_impl = request_impl, **params)
    
        try:
            request.execute()
            ret = HTTP_OK
        except IOError:
            ret = HTTP_OK
        except exception.Abort, e:
            ret = e.aborted_value
            reason = e.reason
        except exception.TopLevelNotFound:
            ret = HTTP_NOT_FOUND
        except exception.ServerError:
            ret = HTTP_INTERNAL_SERVER_ERROR
    
        httpreq.status = ret
        
        return self.do_handle_result(httpreq, ret, reason)


class HTTPRequestImpl(myghty.request.AbstractRequestImpl):
    """subclasses the myghty AbstractRequestImpl class, to provide the connector between 
    the "real" http request (HTTPRequest) and the Myghty request (Request).
    
    this class is responsible for setting up the resources needed by Request, including 
    the output buffer, session, request arguments (usually via a FieldStorage type API),
    sending redirects and handling errors and other exceptions.
    """
    
    def __init__(self, httpreq, logger, errorlogger, 
        request_args = None,
        use_session = False, 
        global_args = None, 
        log_errors = False, output_errors = True, error_handler=None, **params):

        self.httpreq = httpreq
        
        if request_args is None:
            request_args = self.do_make_request_args(httpreq, **params)

        self.request_args = request_args
        
        if not global_args is None:
            # copy the global_args in case they are synonymous with those 
            # given to the interpreter
            self.global_args = global_args.copy()
        else:
            self.global_args = {}

        self.global_args['r'] = self.httpreq

        self.session_args = session.MyghtySessionArgs(**params)

        self.error_handler = error_handler

        self.use_session = use_session
        if self.use_session:
            self.global_args['s'] = self.get_session()

        self.log_errors = log_errors
        self.output_errors = output_errors
        self.logger = logger
        self.errorlogger = errorlogger

        self.out_buffer = self.do_get_out_buffer(httpreq, **params)

    buffer = property(lambda self:self.out_buffer)
    
    def do_make_request_args(self, httpreq, **params):raise NotImplementedError()

    def do_get_out_buffer(self, httpreq, out_buffer = None, **params):raise NotImplementedError()

    def log(self, message):
        self.logger.write(message)
        
    def request_args_from_fieldstorage(self, fields):
        """based on FieldStorage request information, returns a dict of key values, where the values are
        one of:  a string, a list of strings, a Field object with an upload value, a list of
        Field objects with upload values."""

        def formatfield(field):
            if type(field) == types.ListType:
                return map(formatfield, field)
            elif not field.file:
                return field.value
            else:
                return field

        request_args = {}
        
        if fields.file:
            request_args['_file'] = fields.file
        elif fields.list:
            for key in fields.keys():
                request_args[key] = formatfield(fields[key])
        return request_args
        
    def get_session(self, **params):
        if not hasattr(self, 'session'):
            self.session = self.session_args.get_session(self.httpreq, **params)
            
        return self.session
        
    def handle_error(self, error, m, **params):
        if self._run_error_handler(self.error_handler, self.errorlogger, error, m, r = self.httpreq, **params): return

        if isinstance(error, exception.TopLevelNotFound):
            if not error.silent:
                self.errorlogger.write(error.singlelineformat())
            raise error
        else:
            if self.log_errors:
                self.errorlogger.writelines(string.split(error.format(), "\n"))
            else:
                self.errorlogger.write(error.singlelineformat())

            if self.output_errors:
                self.httpreq.content_type = "text/html"
                self.out_buffer.write(error.htmlformat())
                #self.out_buffer.write(error.textformat())
            else:
                raise exception.ServerError(wrapped = error)

        
    def send_redirect(self, path):
        self.httpreq.err_headers_out["Location"] = path
        self.httpreq.content_type = "text/html"
        self.httpreq.status = 302
        self.out_buffer.write('<p>The document has moved'
                  ' <a href="%s">here</a></p>\n'
                  % path)
        raise exception.Redirected(path, 302)

    def send_abort(self, ret, reason):
        raise exception.Abort(ret, reason)
        

    def clone(self, **params):
        # quick shallow copy cloner.  constructor clone doesnt
        # really work in a superclass.  
        # override this if a more complex clone is needed
        cloner = ConstructorClone(self, **params)
        impl = cloner.copyclone()
        return impl
    

class HeaderTable(OrderedDict):
    """a dictionary for storing HTTP headers, works similarly to the table object
    in mod_python."""
    
    def add(self, key, value):
        if self.has_key(key):
            list = self[key]
            if type(list) != types.ListType:
                list = [list]
                self[key] = list
            list.append(value)
        else:
            self[key] = value
    
    def __getitem__(self, key):
        return OrderedDict.__getitem__(self, string.lower(key))

    def __setitem__(self, key, value):
        OrderedDict.__setitem__(self, string.lower(key), value)


    def has_headers(self):
        return len(self) > 0
        
    def get_output(self, buffer):
        for key, value in self.iteritems():
            key = re.sub('_', '-', key)
            if type(value) == types.StringType:
                buffer.write("%s: %s\n" % (key, value))
            else:
                for v in value:
                    buffer.write("%s: %s\n" % (key, v))


class HTTPResolver(myghty.resolver.FileResolver):
    """a subclass of the FileResolver, which can have HTTP-environment-specific
    functionality attached to it if desired."""
    pass
        
class HTTPRequest(object):
    """represents the actual HTTP request.  implementations have to provide this
    object's implementation.  in the case of mod_python, the actual mod_python
    request object is used instead of this one.  the interface should have a 
    compatible subset of the mod_python apache request object's attributes as follows:

    # set up by the base class __init__ method  
    headers_in
    err_headers_out
    headers_out
    status
    headers_sent = False

    # should be added by the implementing __init__ method
    method
    content_type

    # not as required, but preferred:
    path_info
    args
    filename
    
    # required method is:
    do_send_headers()
    
    the headers attributes should use the HeaderTable class to implement.   
    """ 

    def __init__(self):
        self.headers_in = HeaderTable()
        self.headers_out = HeaderTable()
        self.err_headers_out = HeaderTable()
        self.status = HTTP_OK
        self.headers_sent = False
        

    def send_http_header(self):
        if not self.headers_sent:
            self.headers_sent = True
            self.do_send_headers()
            
    def do_send_headers(self):raise NotImplementedError()
    
    def get_response_headers(self):
        if len(self.err_headers_out):
            self.err_headers_out['content-type'] = self.content_type
            return self.err_headers_out
        else:
            self.headers_out['content-type'] = self.content_type
            return self.headers_out
    

class HTTPWriter(myghty.buffer.BufferDecorator):
    """subclasses BufferDecorator to provide the output stream for the response.  

    also calls send_http_header() off of the request at the moment content is first
    sent.
    """

    def __init__(self, httpreq, out_buffer, *args, **params):
        myghty.buffer.BufferDecorator.__init__(self, out_buffer)
        self.httpreq = httpreq
        self.headers_sent = False

    def send_headers(self):
        if not self.headers_sent:
            self.headers_sent = True
            self.httpreq.send_http_header()
            

    def write(self, text):
        if not self.headers_sent:
            self.send_headers()
        self.buffer.write(text)

    def writelines(self, lines):
        if not self.headers_sent:
            self.send_headers()
        self.buffer.writelines(lines)




# what HTTP thingamabob is complete without the HTTP status codes
# defined somewhere
HTTP_CONTINUE                     = 100
HTTP_SWITCHING_PROTOCOLS          = 101
HTTP_PROCESSING                   = 102
HTTP_OK                           = 200
HTTP_CREATED                      = 201
HTTP_ACCEPTED                     = 202
HTTP_NON_AUTHORITATIVE            = 203
HTTP_NO_CONTENT                   = 204
HTTP_RESET_CONTENT                = 205
HTTP_PARTIAL_CONTENT              = 206
HTTP_MULTI_STATUS                 = 207
HTTP_MULTIPLE_CHOICES             = 300
HTTP_MOVED_PERMANENTLY            = 301
HTTP_MOVED_TEMPORARILY            = 302
HTTP_SEE_OTHER                    = 303
HTTP_NOT_MODIFIED                 = 304
HTTP_USE_PROXY                    = 305
HTTP_TEMPORARY_REDIRECT           = 307
HTTP_BAD_REQUEST                  = 400
HTTP_UNAUTHORIZED                 = 401
HTTP_PAYMENT_REQUIRED             = 402
HTTP_FORBIDDEN                    = 403
HTTP_NOT_FOUND                    = 404
HTTP_METHOD_NOT_ALLOWED           = 405
HTTP_NOT_ACCEPTABLE               = 406
HTTP_PROXY_AUTHENTICATION_REQUIRED= 407
HTTP_REQUEST_TIME_OUT             = 408
HTTP_CONFLICT                     = 409
HTTP_GONE                         = 410
HTTP_LENGTH_REQUIRED              = 411
HTTP_PRECONDITION_FAILED          = 412
HTTP_REQUEST_ENTITY_TOO_LARGE     = 413
HTTP_REQUEST_URI_TOO_LARGE        = 414
HTTP_UNSUPPORTED_MEDIA_TYPE       = 415
HTTP_RANGE_NOT_SATISFIABLE        = 416
HTTP_EXPECTATION_FAILED           = 417
HTTP_UNPROCESSABLE_ENTITY         = 422
HTTP_LOCKED                       = 423
HTTP_FAILED_DEPENDENCY            = 424
HTTP_INTERNAL_SERVER_ERROR        = 500
HTTP_NOT_IMPLEMENTED              = 501
HTTP_BAD_GATEWAY                  = 502
HTTP_SERVICE_UNAVAILABLE          = 503
HTTP_GATEWAY_TIME_OUT             = 504
HTTP_VERSION_NOT_SUPPORTED        = 505
HTTP_VARIANT_ALSO_VARIES          = 506
HTTP_INSUFFICIENT_STORAGE         = 507
HTTP_NOT_EXTENDED                 = 510




import zblog.controller
import zblog.util.form as formutil
import zblog.database.mappers as mapper
import zblog.domain.actions as actions
import zblog.controller.manage.blog as manageblog

from zblog.controller import access_control

class Manage(zblog.controller.Controller):

    @access_control(login=True)    
    def __call__(self, m, r, ARGS):
        self.template(m, '/admin/index.myt')


index = Manage()



import zblog.controller
import zblog.util.form as formutil
from zblog.domain.user import User
from zblog.domain.blog import Blog
import zblog.database.mappers as mapper
import zblog.domain.actions as actions
from zblog.controller import access_control

class ManageUser(zblog.controller.Controller):
    @access_control(login=True, action=actions.AdminUsers())
    def ajax_edituser(self, m, ARGS, user_id=None, username=None, form=None):
        if form is None:
            form = self.form(m)
        if username is not None:
            user = User.mapper.get_by(name=username)
            if user is not None:
                form.reflect_from(user)
                form['password_set'].required = False
                form['password_repeat'].required = False
            else:
                form.append_error("Username '%s' not found" % username)
        m.comp('/admin/user.myt:userform', form=form)

    @access_control(login=True, action=actions.AdminUsers())
    def edit_user(self, m, ARGS, user_id=None):
        form = self.form(m)

        if user_id:
            form['password_set'].required = False
            form['password_repeat'].required = False
            
        form.set_request(ARGS, validate=True)
        
        if form['password_set'].value and form['password_set'].value != form['password_repeat'].value:
            form.append_error("Passwords do not match")

        if not user_id:
            existing = User.mapper.get_by(name=form['name'].value)
            if existing is not None:
                form['name'].append_error("Username '%s' already exists" % form['name'].value)
            
        if not form.is_valid():
            self.ajax_edituser(m, ARGS, form=form)
            return

        mapper.begin()
        if user_id:
            created = False
            user = User.mapper.get(user_id)
        else:
            created = True
            user = User()

        form.reflect_to(user)
        mapper.commit()
        form.append_success("User '%s' %s" % (user.name, created and 'created' or 'updated'))
        form.reflect_from(user)
        m.comp('/admin/user.myt:userform', form=form)
    
    @access_control(login=True, action=actions.AdminUsers())
    def delete_user(self, m, ARGS, user_id, confirm=False):
        form = self.form(m)
        user = User.mapper.get(user_id)
        if user is None:
            form.append_error("Userid %d not found" % user_id)
            self.ajax_edituser(m, ARGS, form=form)
            return

        if not confirm:
            m.comp('/admin/user.myt:delete_confirm', user=user)
            return
        name = user.name
        mapper.begin()
        mapper.delete(user)
        mapper.commit()
        form.append_success("User '%s' deleted" % name)
        form.clear()
        m.comp('/admin/user.myt:userform', form=form)
            
    def form(self, m):
        f = formutil.Form('user',[
            formutil.IntFormField('user_id', attribute='id'),
            formutil.FormField('name', required=True),
            formutil.FormField('fullname', required=True),
            formutil.FormField('group', required=True, data=[(x,x) for x in zblog.domain.user.groups]),
            formutil.FormField('password_set', attribute='password', required=True),
            formutil.FormField('password_repeat', attribute=None, required=True)
        ])
        return f

index=ManageUser()        


__all__ = ['Item', 'Variant', 'Category', 'User', 'Cart', 'Address', 'Invoice']

import re
from myghty.util import *

class ModelSupertype(object):pass

class Item(ModelSupertype):
    def __init__(self, name, description, image, price, variants = None):
        self.name = name
        self.description = description
        self.image = image
        self.price = price
        
        self.variants = {}
        if variants is not None:
            for v in variants:
                list = self.variants.setdefault(v.categoryname, OrderedDict())
                list[v.name] = v

    def get_variant_categories(self):
        return self.variants.keys()
        
    def get_variant(self, categoryname, name):
        try:
            return self.variants[categoryname][name]
        except KeyError:
            return None
            
    def __eq__(self, other):
        return other.name == self.name

    def __ne__(self, other):
        return not self.__eq__(other)

class Variant(ModelSupertype):
    def __init__(self, categoryname, name, price = None):
        self.categoryname = categoryname
        self.name = name
        self.price = price
        
    def __eq__(self, other):
        return (
            self.categoryname == other.categoryname
            and self.name == other.name
            )

    def __ne__(self, other):
        return not self.__eq__(other)

class Category(ModelSupertype):
    def __init__(self, name, description, items, parent = None):
        self.name = name
        self.description = description
        self.items = items
        self.subcategories = OrderedDict()
        self.parent = parent
        
        self.path = '/%s' % self.name
        if self.parent is not None:
            self.path = parent.path + self.path
            self.parent.subcategories[self.name] = self
            self.root = self.parent.root
        else:    
            self.allitems = {}
            self.root = self
        
        for item in items:
            if not hasattr(item, 'primarycategory'):
                item.primarycategory = self
                self.root.allitems[item.name] = item

    def get_item(self, name):
        return self.root.allitems[name]
        
    def get_path(self):
        return self.path
        
    def get_category(self, path):
        match = re.match(r'/([^/]*)(/.*)?', path)
        
        (token, remainder) = match.group(1, 2)

        if not token: return self
        
        c = self.subcategories[token]
        
        if not remainder: return c
        else: return c.get_category(remainder)
        
    def get_categories(self):
        return iter(self.subcategories)
            
    def __str__(self):
        return self.name
        
class User(ModelSupertype):
    def __init__(self, username = None, email = None):
        self.username = username
        self.email = email
        self.addresses = []
        self.cart = None
        self.invoice = None

class Cart(ModelSupertype):
    def __init__(self):
        self.items = []
    
    def add_item(self, item, quantity, variants = []):
        for i in self.items:
            if i.equals(item, variants):
                i.quantity += quantity
                return

        self.items.append(CartItem(item, quantity, variants))
    
    def remove_item(self, index):
        self.items[index:index + 1] = []
                                
    def get_total(self):
        sum = 0
        for i in self.items:
            sum += i.get_total_price()
        return sum

    def is_empty(self):
        return len(self.items) == 0
        
class CartItem(ModelSupertype):
    def __init__(self, item, quantity, variants):
        self.item = item
        self.variants = dict([(v.categoryname, v) for v in variants])
        self.quantity = quantity

        self.price = item.price        
        for v in self.variants.values():
            if v.price is not None:
                self.price = v.price

    def get_variant_categories(self):
        return self.variants.keys()
    
    def get_variant(self, categoryname):
        try:
            return self.variants[categoryname]
        except KeyError:
            return None
    
    def get_item_price(self):
        return self.price
        
    def get_total_price(self):
        return self.quantity * self.price
                    
    def equals(self, item, variants):
        if self.item != item:
            return False
            
        for variant in variants:
            try:
                if self.variants[variant.categoryname] != variant:
                    return False
            except KeyError:
                return False
        
        return True
                
    def __eq__(self, other):
        return self.equals(other.item, other.variants.values())

    def __ne__(self, other):
        return not self.__eq__(other)
                
class Address(ModelSupertype):
    def __init__(self, firstname = None, 
               lastname = None, 
               street1 = None, 
               street2 = None, 
               city = None, 
               state = None, 
               zipcode = None, 
               country = None):
        self.street1 = street1
        self.street2 = street2
        self.firstname = firstname
        self.lastname = lastname
        self.city = city
        self.state = state
        self.zipcode = zipcode
        self.country = country
        
    def __eq__(self, other):
        for key in ('firstname', 'lastname', 'street1',
            'street2', 'city', 'state', 'zipcode', 'country'):
            if (not hasattr(other, key) 
                or getattr(other, key) != getattr(self, key)):
                return False
        return True
    
    def __ne__(self, other):
        return not self.__eq__(other)
        

class CreditCard(ModelSupertype):
    def __init__(self, ccname = None, cctype = None, ccnumber = None, ccexp = None):
        self.ccname = ccname
        self.cctype = cctype
        self.ccnumber = ccnumber
        self.ccexp = ccexp
    
    def _hiddencc(self):
        return ((len(self.ccnumber) - 5) * 'X') + self.ccnumber[-4:len(self.ccnumber)]
        
    hiddencc = property(_hiddencc)
    
class Invoice(ModelSupertype):
    def __init__(self, items, user, billingaddress = None, shippingaddress = None, creditcard = None):
        self.items = items
        self.user = user
        self.billingaddress = billingaddress
        self.shippingaddress = shippingaddress
        self.creditcard = creditcard
        
    def get_total(self):
        sum = 0
        for i in self.items:
            sum += i.get_total_price()
        return sum



import zblog.controller
import zblog.util.form as formutil
from zblog.domain.user import User
from zblog.domain.blog import Blog
import zblog.database.mappers as mapper
import zblog.domain.actions as actions
from zblog.controller import access_control

class ManageBlog(zblog.controller.Controller):        
    @access_control(login=True, action=actions.EditBlog())    
    def ajax_editblog(self, m, ARGS, blog_id=None, form=None):
        if form is None:
            form = self.form(m)
        if blog_id:
            blog = Blog.mapper.get(blog_id)
            form.reflect_from(blog)
        m.comp('/admin/blog.myt:blogedit', form=form)

    @access_control(login=True, action=actions.EditBlog())
    def bloglist(self, m):
        m.comp('/admin/blog.myt:bloglist')
        
    @access_control(login=True, action=actions.EditBlog())
    def edit_blog(self, m, ARGS, blog_id=None):
        form = self.form(m)
        form.set_request(ARGS, validate=True)

        if not form.is_valid():
            self.ajax_editblog(m, ARGS, form=form, blog_id=blog_id)
            return

        owner = User.mapper.get_by(name=form['owner_name'].value)
        if owner is None:
            form.append_error("Could not locate user %s" % form['owner_name'].value)
            self.ajax_editblog(m, ARGS, form=form, blog_id=blog_id)
            return
        
        mapper.begin()

        if blog_id:
            created = False
            blog = Blog.mapper.get(blog_id)
        else:
            created = True
            blog = Blog()

        form.reflect_to(blog)
        blog.owner = owner
        mapper.commit()
        form.append_success("Blog '%s' %s" % (blog.name, created and 'created' or 'updated'))
        form.reflect_from(blog)
        m.comp('/admin/blog.myt:blogedit', form=form)

    @access_control(login=True, action=actions.EditBlog())
    def delete_blog(self, m, ARGS, blog_id=None, confirm=False):
        form= self.form(m)
        blog = Blog.mapper.get(blog_id)
        if blog is None:
            form.append_error("No blog found for id '%s'" % blog_id)
            self.ajax_editblog(m, ARGS, form=form)
            return

        if not confirm:
            m.comp('/admin/blog.myt:delete_confirm', blog=blog)
            return
        
        name = blog.name
        mapper.begin()
        mapper.delete(blog)
        mapper.commit()
        form.append_success("Blog '%s' deleted" % name)
        form.clear()
        m.comp('/admin/blog.myt:blogedit', form=form)

            
    def form(self, m):
        u = self.get_user(m)
        
        f = formutil.Form('blog',[
            formutil.IntFormField('blog_id', attribute='id'),
            formutil.FormField('name', required=True),
            formutil.FormField('description', required=True),
            formutil.FormField('owner_name', required=True, default=u.name, getattribute="owner.name")
        ])
        return f


index = ManageBlog()


# encoding: ascii
# component_recoding.py
"""Test handling of components of mixed character encodings.

Also test unicode output from components.
"""
import re, unittest
import testbase
from myghty import escapes

class ComponentRecodingTests(testbase.ComponentTester):
    config = dict(output_encoding="utf_8")
    
    srcFiles = { 'simple.myt':
                 '''# -*- coding: iso-8859-15 -*-
                 <%flags>trim = "both"</%flags>

                 Howdy\xa0there
                 '''
                 }

    def testSimple(self):
        self.runComponent('/simple.myt')
        self.failUnlessEqual(self.output, 'Howdy\xc2\xa0there')

    def testFilter(self):
        """Test that output gets it output all in one whack
        """
        self.runComponent('''# -*- coding: latin1 -*-
            <%flags>
                trim = "both"
            </%flags>
            <%filter>
                return "f(%s)" % f
            </%filter>
            Pre<& /simple.myt &>Post
            ''')
        self.failUnlessEqual(self.output, 'f(PreHowdy\xc2\xa0therePost)')

    def testAutoFlush(self):
        self.runComponent('''
            <%flags>
                autoflush = True
            </%flags>
            Pre
            % m.write(m.request_impl.out_buffer.getvalue())
            Post
            ''')
        self.failUnlessEqual(self.tidyoutput, 'Pre Pre Post')

    def testEncodingError(self):
        self.failUnlessRaisesWrappedError(
            UnicodeEncodeError, self.runComponent, '/simple.myt',
            config={'output_encoding': 'ascii'})
        self.failUnlessEqual(self.output, '')

        self.runComponent('/simple.myt',
                          config={'output_encoding': 'ascii',
                                  'encoding_errors': 'xmlcharrefreplace'})
        self.failUnlessEqual(self.output, 'Howdy&#160;there')

    def testDecodingError(self):
        comp = '''# encoding: ascii
            %m.write("\\xa0")
            '''
        self.failUnlessRaisesWrappedError(UnicodeDecodeError,
                                          self.runComponent, comp)
        self.failUnlessEqual(self.output, '')

    def testValidCharset(self):
        self.failUnlessRaisesWrappedError(
            UnicodeDecodeError,
            self.runComponent,
            '''# encoding: ascii
            % m.write("\\xa0")
            ''',
            config=dict(output_encoding="ascii"))
    

    def testSetOutputEncoding(self):
        self.runComponent("""% m.write(u'\\xa0')
                             % m.set_output_encoding('utf8')
                             % m.write(u'\\xa0')
                             % m.set_output_encoding('ascii', 'replace')
                             % m.write(u'\\xa0')
                             """,
                          config=dict(output_encoding='latin1',
                                      auto_flush=True))
        self.failUnlessEqual(self.output, "\xa0\xc2\xa0?")
            
################################################################
#
# Test various sorts of encoding/decoding errors
#
################################################################



class EncodingErrorsTests(testbase.ComponentTester):
    config = dict(output_encoding="latin1")

    srcFiles = { 'htdocs/badencoding.myt':
                 '''# encoding: ascii
                 % m.write("\\xa0")
                 '''
                 }

    def testDecodeError(self):
        self.failUnlessRaisesWrappedError(
            UnicodeDecodeError,
            self.runComponent, "/badencoding.myt")
        self.failUnlessEqual(self.output, '')

    def testBadSyntax(self):
        # The compiled component contains non-ascii characters,
        # thought the encoding is declared to be ASCII.
        # The Python compiler generates a SyntaxError.
        self.failUnlessRaises(
            SyntaxError,
            self.loadComponent,
            '''# encoding: ascii
            \xa0
            ''')

    srcFiles['badchar.myt'] = (
        '# encoding: ascii\n'
        '% m.write(u"howdy\u20ac")\n') # EURO SIGN - not encodable to latin1

    def testEncodeError(self):
        self.failUnlessRaisesWrappedError(
            UnicodeEncodeError,
            self.runComponent, "/badchar.myt")
        self.failUnlessEqual(self.output, '')

    def testEncodeErrorIgnore(self):
        self.runComponent("/badchar.myt",
                          config={'encoding_errors': 'ignore'})
        self.failUnlessEqual(self.tidyoutput, 'howdy')

    def testEncodeErrorReplace(self):
        self.runComponent("/badchar.myt",
                          config={'encoding_errors': 'replace'})
        self.failUnlessEqual(self.tidyoutput, 'howdy?')

    def testEncodeErrorXmlCharrefReplace(self):
        self.runComponent("/badchar.myt",
                          config={'encoding_errors': 'xmlcharrefreplace'})
        self.failUnlessEqual(self.tidyoutput, 'howdy&#%d;' % 0x20ac)

    def testEncodeErrorHtmlEntityReplace(self):
        self.runComponent("/badchar.myt",
                          config={'encoding_errors': 'htmlentityreplace'})
        self.failUnlessEqual(self.tidyoutput, 'howdy&euro;')

################################################################
#
# Tests that filters get either ASCII strs or unicodes.
# and that they can return any type acceptable to m.write()
#
################################################################

class FilterTests(testbase.ComponentTester):

    config = dict(output_encoding="latin1")
    
    filter_myt = '''# encoding: latin1
        <%args> greeting = "howdy" </%args>
        <%flags> trim = "both" </%flags>
        <%filter> return "f(%s)" % repr(f) </%filter>
        <% greeting %>
        '''

    def testFilterGetsUnicodeEvenIfInputIsAscii(self):
        self.runComponent(self.filter_myt)
        self.failUnlessEqual(self.output, "f(u'howdy')")

    def testFilterGetsUnicode(self):
        # write a non-ascii string, filter should get a unicode
        self.runComponent(self.filter_myt, greeting=u'foo\u20ac')
        self.failUnlessEqual(self.output, "f(u'foo\u20ac')")


class UnicodeOutputFilterTests(FilterTests):
    filter_myt='''# encoding: latin1
        <%args> greeting = "howdy" </%args>
        <%flags> trim = "both" </%flags>
        <%filter> return u"f(%s)" % repr(f) </%filter>
        <% greeting %>
        '''
    
class ObjectOutputFilterTests(FilterTests):
    filter_myt='''# encoding: latin1
        <%args> greeting = "howdy" </%args>
        <%flags> trim = "both" </%flags>
        <%filter>
            class Retval:
                def __init__(self, val):
                    self.val = val
                def __unicode__(self):
                    return unicode(self.val)
            return Retval("f(%s)" % repr(f))
        </%filter>
        <% greeting %>
        '''


class ObjectOutputFilterTests2(FilterTests):
    filter_myt='''# encoding: latin1
        <%args> greeting = "howdy" </%args>
        <%flags> trim = "both" </%flags>
        <%filter>
            class Retval:
                def __init__(self, val):
                    self.val = val
                def __str__(self):
                    return str(self.val)
            return Retval("f(%s)" % repr(f))
        </%filter>
        <% greeting %>
        '''
    
################################################################
#
# Test capture buffers get the right values (either strs
# or unicodes.)
#
################################################################

class TestBuffer(object):
    def __init__(self):
        self.buf = []
    def write(self, text):
        self.buf.append(text)

class CaptureBufferTests(testbase.ComponentTester):

    config = dict(output_encoding='latin1')

    srcFiles = {
        'ascii.myt':   'hello',
        'unicode.myt': r'%m.write(u"\u20ac")',

        'latin1.myt': '''# encoding: latin1
                         hi\xa0there
                         ''',
        }

    def testCapture(self, bufencoding=None):
        buf = TestBuffer()
        if bufencoding:
            buf.encoding = bufencoding
        
        def mycomp(m):
            m.execute_component("/ascii.myt", store=buf)
            m.execute_component("/latin1.myt", store=buf)
            m.execute_component("/unicode.myt", store=buf)
        comp = self.makeModuleComponent(mycomp)

        self.runComponent(comp)

        self.failUnlessEqual(self.output, "")
        expected = [u'hello', u'hi\u00a0there', u'\u20ac']
        self.failUnlessEqual(buf.buf, expected)
        self.failUnlessEqual(map(type, buf.buf), map(type, expected))

    def testCaptureIgnoresAsciiBufferEncodingAttribute(self):
        self.testCapture(bufencoding='ascii')

    def testScomp(self, comp="/ascii.myt", expect=u'hello'):
        result = self.runComponent('''# encoding: utf8
            <%args> comp </%args>
            % return m.scomp(comp)
            ''', comp=comp)
        self.failUnlessEqual(self.output, "")
        self.failUnlessEqual(result, expect)
        self.failUnlessEqual(type(result), type(expect))

    def testScompLatin1(self):
        self.testScomp(comp='/latin1.myt', expect=u'hi\xa0there')

    def testScompUnicode(self):
        self.testScomp(comp='/unicode.myt', expect=u'\u20ac')
        
################################################################
#
# Test closures
#
################################################################


class ClosureTests(testbase.ComponentTester):

    config = dict(output_encoding='utf8')

    def testClosure(self):
        self.runComponent('''# encoding: latin1
            <%closure show_foo>
                <% foo %>
            </%closure>
            % foo = "foo1"
            <& show_foo &>
            % foo = "\xa0foo2".decode("latin1")
            <& show_foo &>
            ''')
        self.failUnlessEqual(self.tidyoutput, "foo1 \xc2\xa0foo2")

    def testClosureWithContent(self):
        self.runComponent('''# encoding: ascii
            <%closure double>
            % for i in 1, 2:
                <% m.content(i=i) %>
            % # end
            </%closure>
            <&| double &>
                cow<% u"\u20ac" %><% m.content_args["i"] %>
            </&>
            ''')
        self.failUnlessEqual(self.tidyoutput,
                             "cow\xe2\x82\xac1 cow\xe2\x82\xac2")

################################################################
#
# Output escaping.
#
################################################################

class OutputEscapingTest(testbase.ComponentTester):
    config = dict(escapes={'H': escapes.html_entities_escape})

    def do_test(self, escape, expected):
        self.runComponent('''
            <%% u"\\u20ac" | %(escape)s %%>
            <%% "\'" | %(escape)s %%>
            ''' % dict(escape=escape),
            config={'encoding_errors': 'backslashreplace'})
        
        self.failUnlessEqual(self.tidyoutput, expected)

    def test_h(self):
        self.do_test('h', r"\u20ac '")
    def test_u(self):
        self.do_test('u', r"%E2%82%AC %27")
    def test_x(self):
        #self.do_test('x', r"\u20ac &apos;")
        self.do_test('x', r"\u20ac &#39;")
    def test_H(self):
        self.do_test('H', r"&euro; '")

################################################################
#
# Test aborting
#
################################################################

class AbortTest(testbase.ComponentTester):
    def testAbort(self):
        self.runComponent(r'''
            <%flags> autoflush = True </%flags>

            <%def subcomp>
              <%flags> autoflush = False </%flags>
              output
            % m.flush_buffer()
              a mistake
            % m.abort()
            </%def>

            head
            <& subcomp &>
            tail
            ''')
        self.failUnlessEqual(self.tidyoutput, 'head output')

    def testFail(self):
        class MyAbort(Exception): pass
        
        from myghty.request import DefaultRequestImpl
        class MyRequestImpl(DefaultRequestImpl):
            def send_abort(self, code, reason):
                assert code == 404
                assert reason == "not found"
                raise MyAbort(code, reason)
        
        request_impl=MyRequestImpl(out_buffer=self.outputBuffer)

        self.failUnlessRaises(
            MyAbort,
            self.runComponent,
            '''
            head
            % m.flush_buffer()
            body
            % m.abort(404, "not found")
            tail
            ''',
            config=dict(request_impl=request_impl))
        
        self.failUnlessEqual(self.tidyoutput, "head")

################################################################
#
# Test m.send_redirect()
#
################################################################

class RedirectTest(testbase.ComponentTester):

    srcFiles = { 'comp2.myt': """Comp2 here.""" }
    
    def testSoftRedirect(self):
        self.runComponent(r'''
            <%flags> autoflush = False </%flags>
            Leading junk (should be discarded)
            % m.send_redirect("/comp2.myt", hard=False)
            Trailing junk (should be disregarded)
            ''')
        self.failUnlessEqual(self.tidyoutput, 'Comp2 here.')

    def testHardRedirect(self):
        class MyRedirect(Exception): pass
        
        from myghty.request import DefaultRequestImpl
        class MyRequestImpl(DefaultRequestImpl):
            def send_redirect(self, path):
                assert path == '/other.html'
                raise MyRedirect(path)
        
        request_impl=MyRequestImpl(out_buffer=self.outputBuffer)

        self.failUnlessRaises(
            MyRedirect,
            self.runComponent,
            '''
            head
            % m.send_redirect("/other.html")
            tail
            ''',
            config=dict(request_impl=request_impl))
        
        self.failUnlessEqual(self.tidyoutput, "")

################################################################
#
# Test that cache_self output is stored in encoding neutral manner
#
################################################################

class CacheSelfTests(testbase.ComponentTester):

    config = dict(output_encoding='ascii')
    
    srcFiles = {'htdocs/cache_money.myt':
                '''# encoding: latin1
                <%flags> use_cache = True </%flags>
                # A random number of EUROs, with a decoding error
                % import random
                <% u"\\xa4" %><% str(random.randint(1,1000)) %>
                '''
                }

    setup_done = False
    
    def class_set_up(self):
        if not self.setup_done:
            testbase.ComponentTester.class_set_up(self)
            self.setUp()
            self.runComponent('<& cache_money.myt &>',
                              config=dict(encoding_errors='replace'))
            self.__class__.euros = int(re.search(r'\d+', self.output).group())
            self.tearDown()
            self.__class__.setup_done = True
        
    def testLenient(self):
        self.runComponent('<& cache_money.myt &>',
                          config=dict(encoding_errors='replace'))
        self.failUnlessEqual(self.tidyoutput, '?%d' % self.euros)

    def testStrict(self):
        self.failUnlessRaisesWrappedError(
            UnicodeEncodeError,
            self.runComponent, '<& cache_money.myt &>',
            config=dict(encoding_errors='strict'))
                                           
################################################################
#
# Test that trimming filter works with either unicode or str input
#
################################################################

class TrimTests(testbase.ComponentTester):

    config = dict(output_encoding='utf8')
    
    def testStrStrip(self):
        self.runComponent(
            '''# encoding: ascii
            <%flags> trim = "both" </%flags>
            foo
            ''')
        self.failUnlessEqual(self.output, 'foo')
        self.failUnlessEqual(type(self.output), str)

    def testUnicodeStrip(self):
        self.runComponent(
            '''# encoding: latin1
            <%flags> trim = "both" </%flags>
            \xa0 bar
            % m.write(u"\\u20ac")
            ''')
        self.failUnlessEqual(self.output, 'bar\n\xe2\x82\xac')

################################################################
#
# Test captured subrequest can have their own output_encoding
#
################################################################

class SubrequestTest(testbase.ComponentTester):

    config = dict(output_encoding='utf-8')

    srcFiles = {'htdocs/subreqbody.myt': r'% m.write(u"\u20ac")'}
    
    def testCanSetOutputEncoding(self):
        import StringIO
        buf = StringIO.StringIO()
        self.runComponent(
            '''# encoding: ascii
            <%args> buf </%args>
            <%init>
                subreq = m.create_subrequest("subreqbody.myt",
                                             out_buffer=buf,
                                             output_encoding="iso-8859-15")
                subreq.execute()                             
            </%init>
            ''',
            buf = buf)
        self.failUnlessEqual(buf.getvalue(), "\xa4")
        self.failUnlessEqual(self.output, '')

    def testCurrentEncoding(self):
        # If not output_encoding is specified for capture buffer, it should
        # default to the current value of m.encoding (not m.output_encoding).
        self.failUnlessRaisesWrappedError(
            UnicodeEncodeError,
            self.runComponent,
            '''# encoding: ascii
            <%init>
                import StringIO
                subreq = m.create_subrequest("subreqbody.myt",
                out_buffer=StringIO.StringIO())
                subreq.execute()                             
            </%init>
            ''')

        
    
if __name__ == '__main__':
    unittest.main()



__all__ = ['Blog', 'Post', 'Topic', 'TopicAssociation', 'Comment']

import datetime

class Blog(object):
    def __init__(self, owner=None):
        self.owner = owner
        
class Post(object):
    def __init__(self, user=None, headline=None, summary=None):
        self.user = user
        self.datetime = datetime.datetime.today()
        self.headline = headline
        self.summary = summary
        self.comments = []
        self.comment_count = 0
        
class Topic(object):
    def __init__(self, keyword=None, description=None):
        self.keyword = keyword
        self.description = description

class TopicAssociation(object):
    def __init__(self, post=None, topic=None, is_primary=False):
        self.post = post
        self.topic = topic
        self.is_primary = is_primary
              
class Comment(object):
    def __init__(self, subject=None, body=None):
        self.subject = subject
        self.datetime = datetime.datetime.today()
        self.body = body
        
        



# $Id: interp.py 2133 2006-09-06 18:52:56Z dairiki $
# interp.py - interprets requests to Myghty templates
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
# Original Perl code and documentation copyright (c) 1998-2003 by Jonathan Swartz. 
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#


"""The interp package and its primary class, Interpreter, represents the runtime environment 
for Myghty components.  It serves as a "home base" for configuration information and request
construction, and also assists the Request object in locating components.  The Interpreter
maintains a pool of component objects and handles, at a high level, their compilation 
and filesystem representation."""

from myghty.util import *
import myghty.util as util
from myghty.synchronization import *
import myghty.exception as exception
import myghty.resolver
import myghty.request
import myghty.csource
import myghty.compiler
import myghty.escapes
import myghty.importer as importer
import myghty.buffer
import myghty.cache
import myghty.component
from stat import *
import time, os, stat, re, types, sys, string, random
import posixpath as unixpath
import weakref, imp, py_compile

__all__ = ['Interpreter']

class Interpreter:
    """Represents an environment in which to create requests, compile and execute components."""
    def __init__(self, 
            attributes = None, 
            data_dir = None, 
            compiler = None, 
            resolver = None, 
            request = None,
            code_cache_size = 16777216,
            use_auto_handlers = True,
            use_object_files = True,
            debug_file = myghty.buffer.LinePrinter(sys.stderr),
            debug_threads = False,
            debug_elements = [],
            escapes = {},
            global_args = None,
            auto_handler_name = 'autohandler',
            **params):
        """constructs a new Interpreter object.  All Myghty configuration parameters are sent to this
        method as keyword parameters.  Those parameters that are used directly by this Interpreter are
        consumed.  Remaining arguments are sent off to other constructed objects, which include a
        Compiler, a Resolver, and a Request prototype object."""
        self.global_args = global_args or {}
        self._attributes = {}
        if attributes is not None:
            self._attributes.update(attributes)
            
        self.attributes = InheritedDict(self._attributes, lambda: None)
    
        self.init_params = params
        self.use_auto_handlers = use_auto_handlers
        self.auto_handler_name = auto_handler_name
        self.use_object_files = use_object_files

        self.use_static_source = params.get('use_static_source', False)
    
        # set up debug stuff
        self.debuggers = {}

        self.log_file = myghty.buffer.LogFormatter(debug_file, 'Myghty', id_threads = debug_threads)
        if len(debug_elements):
            self._debug = True

            for elem in debug_elements:
                # how very strange this is !  
                s = ( 
                "def log(msg):\n" +
                "    self.log_file.write('[%s] ' + msg)\n" +
                "self.debuggers[elem] = log") % elem
                exec s in locals()
        
            self.log_file.write("%s starting - debug logging: %s" % (repr(self),  string.join(debug_elements, ' ')))
        else:
            self._debug = False
        # ------------------

        if params.has_key('component'):
            del params['component']

        # set up cache object creator
        self.cache_args = myghty.cache.CacheArgs(cache_data_dir = data_dir, **params)
        if self.debuggers.get('cache', None) is not None:
            self.cache_args.set_params(debug_file = myghty.buffer.FunctionBuffer(self.debuggers['cache']))

        # set up resolver
        if resolver:
            self.resolver = resolver
        else:
            self.resolver = myghty.resolver.Resolver(**params)
        
        if self.debuggers.get('resolution', None) is not None:
            self.resolver.debug_file = myghty.buffer.FunctionBuffer(self.debuggers['resolution'])

        # set up thread-local compiler prototype        
        if not compiler: compiler = myghty.compiler.Compiler(**params)
        self.compiler = ThreadLocal(creator = lambda: compiler.clone())
                
        # set up request prototype
        if request:
            self.request = request
        else:
            self.request = myghty.request.Request(self, **params)


        # data directory stuff, make sure directories exist     
        if data_dir is not None:
            self.data_dir = data_dir
            self.object_dir = unixpath.join(data_dir, 'obj')
            self.lock_dir = unixpath.join(data_dir, 'lock')
        else:
            self.data_dir = None
            self.object_dir = None
            self.lock_dir = None
        
        if data_dir is None:
            self.use_object_files = False
        else:
            self._verify_directory(self.object_dir)
            self._verify_directory(self.lock_dir)

        # a dictionary of CompileRecord objects linked to the filename they
        # are compiled into.  strong reference provided by the resulting FileComponent
        # object.
        self.reverse_lookup = weakref.WeakValueDictionary()
        
        def componentdeleted(comp):
            if self._debug: self.debug("code_cache size %d - removing %s (%d bytes)" % (self.code_cache.currentsize, comp.id, comp.size), "codecache") 

        self.code_cache = LRUCache(code_cache_size, deletefunc=componentdeleted)

        self.escapes = {'h':myghty.escapes.html_escape, 'u':myghty.escapes.url_escape, 'x':myghty.escapes.xml_escape}
        self.escapes.update(escapes)


    
    def execute(self, component, **params):
        """executes the given component with the given request construction arguments.
        
        The component can be a string path which will be resolved, or a Component object."""
        return self.make_request(component = component, **params).execute()

    def make_request(self, component, **params):
        """creates a new request with the given component and request construction arguments.
        
        The request is copied from an existing request instance, overriding its parameters
        with those given.  The returned request is executed by calling its execute() method.
        
        The component can be a string path which will be resolved when the request is 
        executed, or a Component object."""
        return self.request.clone(component = component, interpreter = self, **params)

    def raise_error(self, error):   
        """raises an Interpreter error with the given message string."""
        raise exception.Interpreter(error)      


    def component_exists(self, path, **params):
        """returns True if the given path can be resolved to a ComponentSource instance,
        i.e. if a component can be loaded for this path."""
        if self.resolver.resolve(path, raise_error = False, **params):
            return True
        else:
            return False


    def load(self, path, **params):
        """resolves and loads the component specified by the given path.  **params 
        is a set of keyword arguments passed to the resolution system."""
        resolution = self.resolve_component(path, **params)
        if not resolution: return None
        return self.load_component(resolution.csource)
        
    def load_module_component(self, raise_error = True, **params):
        """resolves and loads a module component specified within the given keyword 
        parameters.  The keyword parameters are passed directly to 
        myghty.csource.ModuleComponentSource.  The easiest way to use this method
        is to pass the keyword parameter "arg", which references a string in the form 
        "<modulename>:<callable_or_class>", where <modulename> is a regular Python module
        to be loaded via __import__, and <callable_or_class> is the dotted path
        to either a callable object inside the module or a class inside the module which
        subclasses myghty.component.ModuleComponent."""
        csource = myghty.csource.ModuleComponentSource(**params)
        return self.load_component(csource)

    def resolve_component(self, path, **params):
        """resolves the component specified by the given path.  a myghty.resolver.Resolution
        object is returned containing a myghty.csource.ComponentSource reference.
        Raises ComponentNotFound by default if the path cannot be resolved.  **params is a
        set of keyword parameters sent directly to the resolve() method on myghty.resolver.Resolver."""
        return self.resolver.resolve(path, **params)
        
    def load_component(self, csource):
        """loads the component corresponding to the given ComponentSource object.  This can 
        be any subclass of ComponentSource.  If the component was already loaded and exists
        within this Interpreter's code cache, returns the existing Component.  If the Component
        has been modified at its source, it will re-load or re-compile the Component as needed.
        If this Component has not been loaded, it will import or compile the Component as needed.
        This operation is also synchronized against other threads.  On systems that support
        file-locking it is synchronized against other processes as well.
        """
        component_id = csource.id
        try:
            component = self.code_cache[component_id]

            if  (
                self.use_static_source or
                (component.component_source.last_modified >= csource.last_modified and
                component.load_time >= csource.last_modified)
                ):
                return component
            else:
                component.needs_reload = True

        except KeyError: pass

        # lock this block to other threads based on the ID of the component
        name_lock = self._get_component_mutex(component_id)
       
 
        def create():
            component = self.get_compiled_component(csource, use_file = self.use_object_files, recompile_newer = not self.use_static_source)
            if self._debug: self.debug("code_cache size %d - adding %s (%d bytes)" % (self.code_cache.currentsize, component.id, component.size), "codecache")
            component.load_time = int(time.time())
            component.needs_reload = False
            return component

        def isvalid(component):
            return not component.needs_reload
    
        component = self.code_cache.sync_get(component_id, create, name_lock, isvalid)
        return component
                
    def _get_component_mutex(self, component_id):
        return NameLock(identifier = "interpreter/loadcomponent/%s" % component_id)

    def make_component(self, source, id = None):
        """creates an in-memory template Component given its source as a string...this can
        be any Myghty template that would ordinarily be compiled from the filesystem.  
        the optional "id" string parameter is used to identify this component within 
        this Interpreter's code cache.  If not given, one will be generated."""
        csource = myghty.csource.MemoryComponentSource(source = source, id = id)
        return self.get_compiled_component(csource, use_file = False)
        
    def get_component_object_files(self, component_source):
        """returns a tuple containing the full paths to the .py, .pyc, and .pyo files corresponding to the
        given FileComponentSource object."""
        return (
                self.object_dir + '/' + component_source.path_id + '/' + component_source.path + ".py",
                self.object_dir + '/' +  component_source.path_id + '/' + component_source.path + ".pyc",
                self.object_dir + '/' +  component_source.path_id + '/' + component_source.path + ".pyo"
            )
            


    class CompileRecord:
        """a record used to link exception reporting back to the originating python
        and myghty template source"""
        
        def __init__(self, module_id, csource, compiledto = None, compiledsource = None):
            self.module_id = module_id
            self.csource = csource
            self.compiledto = compiledto
            self.compiledsource = compiledsource

        def get_compiled_source(self):
            if self.compiledsource is not None:
                return util.StringIO(self.compiledsource)
            else:
                return open(self.compiledto)
                
    def get_compiled_component(self, csource, always_recompile = False, recompile_newer = True, use_file = True):
        """used by load_component to deliver a new Component object from a ComponentSource."""
        if not csource.can_compile():
            return self._get_module_component(csource, always_recompile, recompile_newer)
        elif not use_file:
            return self._get_inmemory_component(csource, always_recompile, recompile_newer)
        else:
            return self._get_file_component(csource, always_recompile, recompile_newer)

    def _get_module_component(self, csource, always_recompile = False, recompile_newer = True):
        """loads a module component from a regular Python module.  
        Most of the module-loading code is in the util package."""
        
        module = csource.module
        if self._debug: self.debug("loading component %s from module object %s" % (csource.id, repr(module)), "classloading")

        if module.__name__ != '__main__':
            if recompile_newer or always_recompile: 
                csource.reload(importer.reload_module(module))
           
        if csource.class_ is not None and issubclass(csource.class_, myghty.component.ModuleComponent):
            return csource.class_(self, csource)
        else:
            return myghty.component.FunctionComponent(self, csource)

            
    def _get_file_component(self, csource, always_recompile = False, recompile_newer = True):
        """loads and compiles a file-based component as a regular Python module with a .py and .pyc file."""

        destbuf = None
    
        compiler = self.compiler.get()

        # the full module name of the component to be compiled
        cid = "_myghtycomp_" + re.sub(r"\W", "_", csource.path_id) + "_" + re.sub(r"\W", "_", csource.id)

        (object_file, compiled_object_file, optimized_object_file) = self.get_component_object_files(csource)

        # since we might write to a file in this area, synchronize against other threads and processes
        file_name_lock = Synchronizer("interpreter/filewriter/%s" % csource.id, use_files = True, lock_dir = self.lock_dir)

        def needs_recompile():
            # the _need_file checks the csource last modified, which corresponds to the .myt
            # file's last modified, to the actual .py file's modified time.
            return (
                always_recompile or
                (recompile_newer and self._need_file(csource.last_modified, object_file)) or
                (not self._file_exists(object_file))
                or (hasattr(csource, '_bad_magic_number') and csource._bad_magic_number)
                )

        force_reload = False
        
        # check if we have to recompile to a file
        if needs_recompile():

            file_name_lock.acquire_write_lock()
            try:
                # through the lock, check again to see if another fork already compiled
                if needs_recompile():

                    self._verify_directory(unixpath.join(self.object_dir, "./" +  csource.path_id + "/" + csource.dir_name))

                    if self._file_exists(compiled_object_file):
                        os.remove(compiled_object_file)
                    if self._file_exists(optimized_object_file):
                        os.remove(optimized_object_file)

                    if self._debug: self.debug("compiling %s in file %s" % (csource.id, object_file), "classloading")

                    destbuf = file(object_file, 'w')

                    csource.get_object_code(compiler, destbuf)

                    destbuf.close()

                    if getattr(csource, '_bad_magic_number', False):
                        force_reload = True
                    csource._bad_magic_number = False
            finally:
                file_name_lock.release_write_lock()

        file_name_lock.acquire_read_lock()
        try:

            comprec = Interpreter.CompileRecord(cid, csource, compiledto = object_file)
            self.reverse_lookup[object_file] = comprec

            if self._debug: self.debug("loading component from source file %s" % object_file, "classloading")
            module = importer.filemodule(object_file, id = cid, forcereload = force_reload)
            csource.module = module

            # attach the compile record to the module so it remains
            # referenced as long as the module does
            module.__compile = comprec
            
            # get the size of the source file and attach to the component source
            csource.modulesize = os.stat(object_file)[ST_SIZE]

            # check for bad magic number (incompatible compiler version), 
            # and recompile if needed
            if (
                not hasattr(module, '_MAGIC_NUMBER') or
                module._MAGIC_NUMBER != compiler.get_magic_number()
                ):

                # paranoid check against impossible endless loop
                if hasattr(csource, '_bad_magic_number') and csource._bad_magic_number:
                    raise "assertion failed: csource magic number check completed but recompile still produces bad magic number"

                # set a flag, we'll call again
                csource._bad_magic_number = True
            else:
                csource._bad_magic_number = False
        finally:
            file_name_lock.release_read_lock()

        if csource._bad_magic_number:
            return self.get_compiled_component(csource, always_recompile = always_recompile, recompile_newer = recompile_newer, use_file = True)
        else:
            return module.get_component(self, csource)
                    
    def _get_inmemory_component(self, csource, always_recompile = False, recompile_newer = True):
        """loads and compiles a file-based component as an in-memory module."""
        destbuf = None
    
        compiler = self.compiler.get()

        # the full module name of the component to be compiled
        cid = "_myghtycomp_" + re.sub(r"\W", "_", csource.path_id) + "_" + re.sub(r"\W", "_", csource.id)

        if self._debug: self.debug("compiling %s in memory" % csource.id, "classloading")
        destbuf = util.StringIO()
        csource.get_object_code(compiler, destbuf)

        filename = "memory:" + cid

        module = csource.module
            
        def needs_recompile():
            return (
                always_recompile or
                module is None 
                or 
                (recompile_newer and csource.last_modified > module._modified_time)
                )

        if needs_recompile():
            module = imp.new_module(cid)
            code = compile(destbuf.getvalue(), filename, 'exec')
            exec code in module.__dict__, module.__dict__
            module._modified_time = time.time()
            csource.module = module

        comprec = Interpreter.CompileRecord(cid, csource, compiledsource = destbuf.getvalue())
        self.reverse_lookup[filename] = comprec

        module.__compile = comprec
        csource.modulesize = len(destbuf.getvalue())
        
        csource._bad_magic_number = False

        return module.get_component(self, csource)

    def _file_exists(self, file):           
        return os.access(file, os.F_OK)
    
    def _need_file(self, modified_time, file):
        return (not os.access(file, os.F_OK) or
            modified_time > os.stat(file)[ST_MTIME] or
            os.stat(file)[ST_SIZE] == 0)

    def _verify_directory(self, dir):
        verify_directory(dir)


    def debug(self, message, type = None):
        """writes a debug message to the debug file.  the optional type string refers to a "debug_element" type,
        i.e. 'resolution', 'cache', etc. which will only write a message if that debug_element was 
        enabled with this Interpreter."""
        if type is None:
            self.log_file.write(message)
        else:
            log = self.debuggers.get(type, None)
            if log is not None: log(message)

    # deprecated 
    def get_attribute(self, key):
        """gets an attribute from this Interpreter.  deprecated, use the "attributes"
        member directly instead."""
        return self.attributes(key)
        
    def set_attribute(self, key, value):
        """sets an attribute on this Interpreter.  deprecated, use the "attributes"
        member directly instead."""
        self.attributes(key, value)

    def get_attributes(self):
        """returns the attributes dictionary for this Interpreter.  
        deprecated, use the "attributes" member directly instead."""
        return self.attributes()



import myghty.component
import myghty.request as request
import highlight
import os
import posixpath as unixpath

"""module components for source code viewing, path translation dhandler.

this package illustrates how to create small template-based components 
within a module component, and also how to write a dhandler as a module 
component."""

class ViewSource(myghty.component.ModuleComponent):
    def do_component_init(self, **params):
        comp = """
            <%global>
                import posixpath as unixpath
            </%global>
            <%args>
                content
                name
                path
                uri
            </%args>

            <%method title>
                <%args scope="subrequest">
                name
                </%args>
                Source of <% name %>
            </%method>
            
            <span class="smallheader">Source of <a href="<% uri %>"><% path %>/</a><% name %></span>
<a href="<% name %>?type=plain">(download)</a>

<pre class="source">
<% content %>
</pre>
        """
        
        dirlist = """
        <%args>
            directories
            files
            name
            binfiles
            parent
            path
        </%args>

            <%method title>
                <%args scope="subrequest">
                name
                </%args>
                Listing of <% name %>
            </%method>

            <div class="smallheader">Source Code Listing of <a href="<% parent %>"><% path %>/</a><% name %>/</div>
            
            <br/>
    <div class="filelist">
    <a href="<% parent %>">(parent directory)</a><br/>
%    for directory in directories:
    <a href="<% directory %>/index"><% directory %>/</a><br/>
%

%    for file in files:
    <a href="<% file %>"><% file %></a><br/>
%

%    for file in binfiles:
    <% file %><br/>
%

    </div>
"""
        self.viewsource = request.instance().interpreter.make_component(comp)
        self.dirlisting = request.instance().interpreter.make_component(dirlist)
        
    def do_run_component(self, m, r, ARGS, **params):
        filename = r.filename

        source_uri = m.interpreter.attributes['source_uri']

        if filename.endswith('/index'):
            (filename, index) = unixpath.split(filename)
        
        fileuri = filename[len(unixpath.commonprefix([m.interpreter.attributes['source_root'], filename])):]

        if os.path.isdir(filename):
            listing = os.listdir(filename)
            files = filter(lambda f: not f.endswith(".pyc") and os.path.isfile(os.path.join(filename, f)), listing)
            binfiles = filter(lambda f: f.endswith(".pyc") and os.path.isfile(os.path.join(filename, f)), listing)
            directories = filter(lambda f: os.path.isdir(os.path.join(filename, f)), listing)
            (path, name) = unixpath.split(fileuri)
            (parent, n2) = unixpath.split(m.get_request_path())
            parent = unixpath.join(parent, '..', 'index')    
            m.subexec(self.dirlisting, directories = directories, files = files, binfiles = binfiles, path=path, name=name, parent = parent)
        else:
            try:
                f = file(filename)
            except IOError:
                m.abort(404)
    
            r.content_type = 'text/html'        
            s = f.read()
   
            if ARGS.get('type', None) == 'plain':
                 r.content_type = 'text/plain'
                 m.write(s)
                 return
 
            s = highlight.highlight(s, filename = fileuri)
    
            (path, name) = unixpath.split(fileuri)
            (uri, n2) = unixpath.split(m.get_request_path())
            uri = unixpath.join(uri, 'index')    
            m.subexec(self.viewsource, content = s, uri = uri, path = path, name=name)





"""global database stuff.  includes functions for finding out about database drivers, as well as the global SQLEngine used by the application."""
import zblog
import sqlalchemy.engine
import sys

def dbtypes():
    """returns a listing of SQLEngine descriptors for usage in bootstrap system"""
    return sqlalchemy.engine.engine_descriptors()
    
def get_descriptor(name):
    """returns the named SQLEngine descriptor for usage in bootstrap system."""
    for desc in dbtypes():
        if desc['name'] == name:
            break
    else:
        desc = None
    return desc
    
def test_connection(name, kwargs):
    """given a SQLEngine name and keyword arguments, creates a connection, or 
    propigates whatever exceptions occur."""
    e = sqlalchemy.engine.create_engine(name, kwargs)
    e.connection()

def init_engine():
    global engine
    conf = zblog.config['database'].copy()
    driver = conf.pop('driver')
    engine = sqlalchemy.engine.create_engine(driver, conf, echo=conf.pop('echo', False))
    # load mapper module to intialize mapper attributes on domain classes
    __import__('zblog.database.mappers')
    
# add init_engine to startup callables
zblog.startup.append(init_engine)

def init_database(admin_username, admin_password, logger):
    """creates database tables and inserts administrative user upon installation."""
    import zblog.database.tables
    if zblog.database.tables.db != engine:
        reload(zblog.database.tables)
    e = engine.echo        
    engine.echo=True
    engine.logger=logger
    zblog.database.tables.create_tables()
    import zblog.domain.user as user
    import zblog.database.mappers as mapper
    mapper.begin()
    try:
        u = user.User(admin_username, 'Administrator', admin_password, user.administrator)
        mapper.commit()
    finally:
        engine.echo = e
        engine.logger = sys.stdout


"""test/test_testbase.py

Tests for stuff in test/testbase.py.
"""

import re, sys, unittest
import testbase
from myghty.component import Component

class ComponentTesterTests(testbase.ComponentTester):
    """Test cases for ComponentTester
    """
    
    srcFiles = { 'htdocs/foo.myt': 'foo',
                 'lib/barmod.py': 'def f(m): m.write("bar")',
                 }
    config = dict(module_components=[{'bar': 'barmod:f'}])
    
    def testFoo(self):
        self.runComponent("foo.myt")
        self.failUnlessEqual(self.output, 'foo')

    def testBar(self):
        self.runComponent("bar")
        self.failUnlessEqual(self.output, 'bar')

    def testRunInlineComponent(self):
        self.runComponent(
            """# comment
            howdy
            """)
        self.failUnlessEqual(self.tidyoutput, 'howdy')

    def testRunInlineModuleComponent(self):
        def comp(m):
            m.write("output")
        self.runComponent(comp)
        self.failUnlessEqual(self.output, 'output')

    def testMakeModule(self):
        testmod = self.makeModule("def f(m): m.write('boo')")
        self.runComponent(testmod.f)
        self.failUnlessEqual(self.output, "boo")

    def testMakeMemoryComponent(self):
        comp = self.makeMemoryComponent(
            '''
            <%flags>
                trim = "both"
            </%flags>
            from memory
            ''')
        self.failUnless(isinstance(comp, Component))
        self.runComponent(comp)
        self.failUnlessEqual(self.output, "from memory")


class TestUnwrapErrors(testbase.ComponentTester):
    """Test advice added to unittest.TestResult.addError()

    Exceptions which are wrapped in a myghty.Error should be
    unwrapped before the error is reported.
    """
    def x_test(self):
        self.runComponent("""% message = 'Boo'
                             % raise RuntimeError, message""")
        print self.output

    def test(self):
        mytest = unittest.defaultTestLoader.loadTestsFromName(
            '%s.TestUnwrapErrors.x_test' % __name__)
        result = unittest.TestResult()
        mytest.run(result)
        
        self.failUnlessEqual(len(result.errors), 1)
        failed, message = result.errors[0]
        last_line = message.rstrip().split('\n')[-1]
        self.failUnlessEqual("RuntimeError: Boo", last_line)
        
if __name__ == '__main__':
    unittest.main()



#!/usr/local/bin/python

"""myghty HTTPServerHandler runner."""

import myghty.http.HTTPServerHandler as HTTPServerHandler
import sys, os, re
from myghty.resolver import *

root = os.path.abspath(os.path.join(os.path.dirname(__file__), '../'))
sys.path.insert(0, os.path.join(root, 'lib'))

interpreter_config = {
    'root' : root
}
execfile(os.path.join(root, 'config/server_config.py'), globals(), interpreter_config)
interp = HTTPServerHandler.HSHandler(**interpreter_config)

port = 8080
httpd = HTTPServerHandler.HTTPServer(
    port = port,
    
    handlers = [
        {r'.*(?:/|\.myt)$' : interp},
    ],

    docroot = [{'.*' : os.path.join(root, 'htdocs')}],
    
)       
        
print "Listening on port %d" % port
httpd.serve_forever()




import myghty.component as component


# the generate_form module component          
def generate_form(m, form):          
    form.accept_visitor(RenderForm(m))


# base class for an element in an HTML form
class FormElement(object):
    def __init__(self):
        pass

    def accept_visitor(self, visitor): 
        pass

# FormElement implementations
class Form(FormElement):

    def __init__(self, name, action, elements):
        self.name = name
        self.action = action
        self.elements = elements

    def accept_visitor(self, visitor):
        visitor.visit_form(self)

class Field(FormElement):

    def __init__(self, name, description, value = None):
        self.name = name
        self.description = description
        self.value = value

class TextField(Field):

    def __init__(self, name, description, size, value = None):
        Field.__init__(self, name, description, value)
        self.size = size

    def accept_visitor(self, visitor):
        visitor.visit_textfield(self)

class SelectField(Field):

    def __init__(self, name, description, options, value = None):
        Field.__init__(self, name, description, value)
        self.options = options
        for o in self.options:
            o.parent = self
            if o.id == self.value:
                o.selected = True

    def accept_visitor(self, visitor):
        visitor.visit_selectfield(self)

class OptionField(FormElement):
    """an <OPTION></OPTION> tag.  contains a parent attribute that points to a <SELECT> field."""

    def __init__(self, id, description):
        self.id = id
        self.description = description
        self.parent = None
        self.selected = False

    def accept_visitor(self, visitor):
        visitor.visit_option(self)

class SubmitField(FormElement):
    """an <INPUT TYPE="submit"> field."""

    def __init__(self, value):
        self.value = value

    def accept_visitor(self, visitor):
        visitor.visit_submit(self)


# defines an interface for an object that can be 
# walked along a FormElement structure
class FormVisitor:
          def visit_form(self, form):pass
          def visit_textfield(self, textfield):pass
          def visit_selectfield(self, selectfield):pass
          def visit_option(self, option):pass
          def visit_submit(self, submit):pass
          
          
          
# subclass of FormVisitor that walks along a FormElement
# structure and renders components in a request output stream
class RenderForm(FormVisitor):
      def __init__(self, m):
              self.m = m

      def visit_form(self, form):
              def formcontent():
                      for element in form.elements:
                              element.accept_visitor(self)

              self.m.execute_component("/examples/formvisitor/formfields.myc:form", args = dict(form = form), content=formcontent)

      def visit_textfield(self, textfield):
              self.m.comp("/examples/formvisitor/formfields.myc:textfield", textfield = textfield)

      def visit_selectfield(self, selectfield):
              def selectcontent():
                      for element in selectfield.options:
                              element.accept_visitor(self)

              self.m.execute_component("/examples/formvisitor/formfields.myc:select", args = dict(select = selectfield), content=selectcontent)

      def visit_option(self, option):
              self.m.comp("/examples/formvisitor/formfields.myc:option", option = option)

      def visit_submit(self, submit):
              self.m.comp("/examples/formvisitor/formfields.myc:submit", submit = submit)


"""controller package - defines Myghty application controllers"""

__all__ = ['Controller', 'access_control']

from sqlalchemy import *

class Controller(object):
    def template(self, m, template, **kwargs):
        m.subexec(template, **kwargs)
    def get_user(self, m):
        s = m.get_session()
        u = s.get('user', None)
        # import_instance assures that the user deserialized from the session
        # is properly present in the current thread's unit of work context.
        return objectstore.import_instance(u)
        



def access_control(action=None, login=False):
    def decorator(func):
        func.action = action
        func.login = login
        func.public = True
        return func
    return decorator


# $Id: WSGIHandler.py 2133 2006-09-06 18:52:56Z dairiki $
# WSGIHandler.py - handles WSGI requests for Myghty
# Copyright (C) 2004, 2005 Michael Bayer mike_mp@zzzcomputing.com
#
# This module is part of Myghty and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
#


import myghty.interp
import myghty.request
import myghty.resolver
import myghty.buffer
import myghty.escapes as escapes
import myghty.session as session
import myghty.exception as exception
import myghty.http.HTTPHandler as HTTPHandler
import myghty.http.HTTPHandler as http
from myghty.util import *
import os, sys, cgi


def handle(environ, start_response, **params):
    return HTTPHandler.handle_http(WSGIHandler, environ = environ, start_response = start_response, **params)

def get_handler(interpreter_name = None, **params):
    return HTTPHandler.get_handler(WSGIHandler, interpreter_name = interpreter_name, **params)

def application(environ, start_response):
    return get_handler(**environ.get('myghty.application', {})).handle(environ, start_response, **environ.get('myghty.request', {}))

class WSGIHandler(HTTPHandler.HTTPHandler):
    def __init__(self, **params):
        HTTPHandler.HTTPHandler.__init__(self, None, myghty.buffer.LinePrinter(sys.stderr), **params)

    def do_get_init_params(self, httpreq, **params):
        return params
    
    def do_get_resolver(self, **params):
        return WSGIResolver(**params)
    
    def do_make_request_impl(self, httpreq, **params):
        return WSGIRequestImpl(httpreq, **params)
        
    def do_get_component(self, httpreq, **params):
        return httpreq.path_info
        
    def do_handle_result(self, httpreq, status_code, reason):
        httpreq.status = status_code
        httpreq.status_message = reason
        httpreq.send_http_header()
        return [httpreq.out_buffer.getvalue()]


    def handle(self, environ, start_response, httpreq = None, interp = None, request_impl = None, component = None, **params):
        if httpreq is None:
            httpreq = WSGIRequest(environ, start_response)

        return HTTPHandler.HTTPHandler.handle(self, httpreq, interp, request_impl, component, **params)


class WSGIRequest(HTTPHandler.HTTPRequest):
    def __init__(self, environ, start_response):
        HTTPHandler.HTTPRequest.__init__(self)
        
        self.start_response = start_response
        self.environ = environ

        for key in environ.keys():
            if key[0:4] == 'HTTP':
                self.headers_in.add(key[5:], environ[key])
                        
        self.content_type = 'text/html'
        self.method = environ['REQUEST_METHOD']
        self.path_info = environ.get('PATH_INFO', '/')
        self.args = environ.get('QUERY_STRING', None)
        self.status_message = None
        self.filename = environ.get('PATH_TRANSLATED', None)

        self.out_buffer = StringIO()
        
        self.fieldstorage = cgi.FieldStorage(
            fp = environ['wsgi.input'],
            environ = environ,
            keep_blank_values = True
        )

    def do_send_headers(self):
        headers = [(key, value) for key, value in self.get_response_headers().iteritems()]
        if self.status_message is None:
            reason = "HTTP return code"
        else:
            reason = self.status_message
        self.start_response(str(self.status) + " " + reason, headers)
        
        
class WSGIResolver(HTTPHandler.HTTPResolver): pass


class WSGIWriter(HTTPHandler.HTTPWriter):pass
        

class WSGIRequestImpl(HTTPHandler.HTTPRequestImpl):
    def __init__(self, httpreq, **params):
        logger = myghty.buffer.LinePrinter(httpreq.environ['wsgi.errors'])
        HTTPHandler.HTTPRequestImpl.__init__(self, httpreq, logger, logger, **params)
        
    def do_make_request_args(self, httpreq, **params):
        return self.request_args_from_fieldstorage(httpreq.fieldstorage)

    def do_get_out_buffer(self, httpreq, out_buffer = None, **params):
        if out_buffer is None:
            return WSGIWriter(httpreq, httpreq.out_buffer)
        else:
            return WSGIWriter(httpreq, out_buffer)


 
